; Copyright (C) 2016 Texas Instruments Incorporated - http:;www.ti.com/
;
; Redistribution and use in source and binary forms, with or without
; modification, are permitted provided that the following conditions
; are met:
;
; Redistributions of source code must retain the above copyright
; notice, this list of conditions and the following disclaimer.
;
; notice, this list of conditions and the following disclaimer in the
; documentation and/or other materials provided with the
; distribution.
;
; Neither the name of Texas Instruments Incorporated nor the names of
; its contributors may be used to endorse or promote products derived
; from this software without specific prior written permission.
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
; LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
; A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
; OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
; SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
; LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
; DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
; (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
; OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;
; 	file:   event_handler.asm
;
; 	brief:
;
;	Version		Description								Author
; 	0.1     	Created									Thomas Mauer
;include section
	.include "include/macros.inc"
	.include "include/icss_regs.inc"
	.include "include/icss_macros.inc"
	.include "rte_appl_defines.inc"

	.global FN_EVENT_HANDLER

FN_EVENT_HANDLER:
	.if $defined (PRU0)	; PRU0 uses bit 30, PRU1 uses bit 31
	QBBC	EVH_DONE, R31, 30
	; load highest priority currently pending interrupt for host interrupt 0
EVENT_HANDLER_RESTART:
	LDI		TEMP_REG_2.w0, ICSS_INTC_HIPIR0
	.else
	QBBC	EVH_DONE, R31, 31
	; load highest priority currently pending interrupt for host interrupt 1
EVENT_HANDLER_RESTART:
	LDI		TEMP_REG_2.w0, ICSS_INTC_HIPIR1
	.endif
	; Store event number in TEMP_REG_4. None of ISRs below should overwrite this register
	LBCO	&TEMP_REG_4, ICSS_INTC, TEMP_REG_2.w0, 4

	QBBS	EVH_DONE, TEMP_REG_4, NONE_HINT_BIT
	; clear event in INTC
	;SBCO	&TEMP_REG_4, ICSS_INTC, ICSS_INTC_SICR, 4

	; event jump table
	QBEQ	EVH_IEP_EVENT_PEND, TEMP_REG_4, SYS_EVT_IEP_TIM_CAP_CMP_PEND
	.if $defined(PRU0)
	QBEQ	EVH_SFD_EVENT_PEND, TEMP_REG_4, SYS_EVT_PRU0_RX_SOF
	QBEQ	EVH_RX_EOF_EVENT_PEND, TEMP_REG_4, SYS_EVT_PRU0_RX_EOF
	.else
	QBEQ	EVH_SFD_EVENT_PEND, TEMP_REG_4, SYS_EVT_PRU1_RX_SOF
	QBEQ	EVH_RX_EOF_EVENT_PEND, TEMP_REG_4, SYS_EVT_PRU1_RX_EOF
	.endif
	; EOF event; Depending on port configuration one or the other will be active per PRU
	QBEQ	EVH_TX_EOF_EVENT_PEND, TEMP_REG_4, SYS_EVT_PORT0_TX_UNDERFLOW
	QBEQ	EVH_TX_EOF_EVENT_PEND, TEMP_REG_4, SYS_EVT_PORT1_TX_UNDERFLOW
	QBEQ	EVH_MDIO_EVENT_PEND, TEMP_REG_4, SYS_EVT_MDIO_MII_LINK1
	QBEQ	EVH_MDIO_EVENT_PEND, TEMP_REG_4, SYS_EVT_MDIO_MII_LINK0
	; TX overflow occured during cut-through
	QBEQ	EVH_TX_OVERFLOW_EVENT_PEND, TEMP_REG_4, SYS_EVT_PORT0_TX_OVERFLOW
	QBEQ	EVH_TX_OVERFLOW_EVENT_PEND, TEMP_REG_4, SYS_EVT_PORT1_TX_OVERFLOW
	QBEQ	EVH_ECAP_TIMER_TICK_PEND, TEMP_REG_4, SYS_EVT_ECAP_PEND

	QBEQ	EVH_PRU1_WRAPAROUND_EVENT_PEND, TEMP_REG_4, PRU0_PRU1_WRAPAROUND_EVENT

	; Received an event which is not registered. Add code below to investigate why this event has been issued
	;QBA 	EVENT_HANDLER_RESTART
	; clear this not registered event via EVH_CLR_INTC_EVENT

EVH_CLR_INTC_EVENT:
	SBCO	&TEMP_REG_4, ICSS_INTC, ICSS_INTC_SICR, 4
	QBA 	EVENT_HANDLER_RESTART

EVH_DONE:
	RET

EVH_SFD_EVENT_PEND:
EVH_TX_OVERFLOW_EVENT_PEND:
EVH_ECAP_TIMER_TICK_PEND:
	QBA		EVH_CLR_INTC_EVENT

EVH_TX_EOF_EVENT_PEND:
	QBA		EVH_CLR_INTC_EVENT

;*****************************
; IEP event handler
;*****************************
EVH_IEP_EVENT_PEND:
	; read IEP event register and clear event.
	LBCO	&TEMP_REG_1, ICSS_IEP, ICSS_IEP_CMP_STATUS_REG, 4
	SBCO	&TEMP_REG_1, ICSS_IEP, ICSS_IEP_CMP_STATUS_REG, 4
	QBBC	EVH_IEP_CMP1, TEMP_REG_1, 0
;EVH_IEP_CMP0:	; IEP CMP0 timer wrap-around
	; send wraparound event to PRU1
	LDI		R31.w0, PRU0_PRU1_WRAPAROUND_EVENT_IRQ
	QBBS	EVH_IEP_CMP0_UPDATE_CNT, RCV_STATUS_REG, RCV_ACTIVE_FLAG
	XIN		RCV_L2_BANK0_ID, &R18.b0, 1
	QBEQ	EVH_IEP_CMP0_UPDATE_CNT, R18.b0, 0
	SET		RCV_STATUS_REG, RCV_STATUS_REG, RCV_TS_HIGH_UPDATE_FLAG
EVH_IEP_CMP0_UPDATE_CNT:
	LBCO	&TEMP_REG_2, ICSS_SHARED_RAM, IEP_CNT_HIGH_PRU0, 4
	ADD		TEMP_REG_2, TEMP_REG_2, 1								;IEP_HIGH_COUNTER++;
	SBCO	&TEMP_REG_2, ICSS_SHARED_RAM, IEP_CNT_HIGH_PRU0, 4
	; do nothing
EVH_IEP_CMP1:
	;QBBC	EVH_IEP_EVENT_DONE, TEMP_REG_1, 1
	; generate host event
	;LDI		R31, PRU_ARM_EVENT0_IRQ
EVH_IEP_EVENT_DONE:
	QBA		EVH_CLR_INTC_EVENT


;*****************************
; Wraparound event handler for PRU1
;*****************************
EVH_PRU1_WRAPAROUND_EVENT_PEND:
	QBBS	EVH_PRU1_WRAPAROUND_EVENT0_UPDATE_CNT, RCV_STATUS_REG, RCV_ACTIVE_FLAG
	XIN		RCV_L2_BANK0_ID, &R18.b0, 1
	QBEQ	EVH_PRU1_WRAPAROUND_EVENT0_UPDATE_CNT, R18.b0, 0
	SET		RCV_STATUS_REG, RCV_STATUS_REG, RCV_TS_HIGH_UPDATE_FLAG
EVH_PRU1_WRAPAROUND_EVENT0_UPDATE_CNT:
	LBCO	&TEMP_REG_2, ICSS_SHARED_RAM, IEP_CNT_HIGH_PRU1, 4
	ADD		TEMP_REG_2, TEMP_REG_2, 1								;IEP_HIGH_COUNTER++;
	SBCO	&TEMP_REG_2, ICSS_SHARED_RAM, IEP_CNT_HIGH_PRU1, 4
EVH_PRU1_WRAPAROUND_EVENT_DONE:
	QBA		EVH_CLR_INTC_EVENT


;*****************************
; Receive EOF event handler
;*****************************
EVH_RX_EOF_EVENT_PEND:
	QBBS	RX_EOF_EVENT, RCV_STATUS_REG, RCV_ACTIVE_FLAG
	LDI		R31.w2, (D_RESET_RXFIFO | D_RX_ERROR_CLEAR)
	LDI		RCV_STATUS_REG, 0
	;LDI		CT_WRITE_POINTER, 0							;Removed 4-7-2016
	QBA		EVH_CLR_INTC_EVENT
RX_EOF_EVENT:
	SET		RCV_STATUS_REG, RCV_STATUS_REG, RCV_EOF_EVENT_FLAG
	QBA		EVH_CLR_INTC_EVENT

;**********************
; MDIO event handler
;**********************
EVH_MDIO_EVENT_PEND:
	; clear MDIO event in MDIO module
	LDI		TEMP_REG_1, 0x03
	SBCO	&TEMP_REG_1, ICSS_MDIO, ICSS_MDIO_LINKINTMASKED, 4

	; get current link information from MDIO module
	LBCO	&TEMP_REG_1, ICSS_MDIO, ICSS_MDIO_LINK, 4
	LDI		TEMP_REG_2.w0, MDIO_PHY_CONFIG_OFFSET
	LBCO	&TEMP_REG_2, ICSS_SHARED_RAM, TEMP_REG_2.w0, 4
	ZERO	&TEMP_REG_3.w0, 2
PM_PHY_0:
	; parse link bit for PHY0
	QBBC	PM_PHY_0_ACT_HIGH, TEMP_REG_3, 16		; 0: active high; 1: active low
	QBBS	PM_PHY_1, TEMP_REG_1, TEMP_REG_2.b0
	QBA		PM_PHY_0_CONFIG
PM_PHY_0_ACT_HIGH:
	QBBC	PM_PHY_1, TEMP_REG_1, TEMP_REG_2.b0
PM_PHY_0_CONFIG:
	SET		TEMP_REG_3.b0, TEMP_REG_3.b0, 0	;
PM_PHY_1:
	; parse link bit for PHY1
	QBBC	PM_PHY_1_ACT_HIGH, TEMP_REG_2, 24		; 0: active high; 1: active low
	QBBS	PM_STORE_PHY_STATUS, TEMP_REG_1, TEMP_REG_2.b1
	QBA		PM_PHY_1_CONFIG
PM_PHY_1_ACT_HIGH:
	QBBC	PM_STORE_PHY_STATUS, TEMP_REG_1, TEMP_REG_2.b1
PM_PHY_1_CONFIG:
	SET		TEMP_REG_3.b1, TEMP_REG_3.b1, 0
PM_STORE_PHY_STATUS:
	SBCO	&TEMP_REG_3.w0, ICSS_SHARED_RAM, PHY_STATUS_OFFSET, 2
	; if both ports are enabled (0101), enable TX_AUTO_FWD for both ports
	; otherwise disable TX_AUTO_FWD for both ports
	LDI		TEMP_REG_1.w0, 0x0101
	QBEQ	BOTH_PHY_ENABLED, TEMP_REG_3.w0, TEMP_REG_1.w0
	LDI32	TEMP_REG_2, 0x00400101
	LDI32	TEMP_REG_3, 0x00400001
	SBCO	&TEMP_REG_2, MII_RT_CFG, ICSS_MIIRT_TXCFG0, 8

BOTH_PHY_ENABLED:
	LDI32	TEMP_REG_2, 0x00400301
	LDI32	TEMP_REG_3, 0x00400201
	SBCO	&TEMP_REG_2, MII_RT_CFG, ICSS_MIIRT_TXCFG0, 8
	QBA		EVH_CLR_INTC_EVENT

