; Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
;
; Redistribution and use in source and binary forms, with or without
; modification, are permitted provided that the following conditions
; are met:
;
; Redistributions of source code must retain the above copyright
; notice, this list of conditions and the following disclaimer.
;
; notice, this list of conditions and the following disclaimer in the
; documentation and/or other materials provided with the
; distribution.
;
; Neither the name of Texas Instruments Incorporated nor the names of
; its contributors may be used to endorse or promote products derived
; from this software without specific prior written permission.
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
; LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
; A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
; OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
; SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
; LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
; DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
; (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
; OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;
; 	file:   main.asm
;
; 	brief:
;
;	Version		Description								Author
; 	0.1     	Created									Thomas Mauer
;   0.2         RTE										Haisem Khan
;include section
	.include "include/macros.inc"
	.include "include/icss_regs.inc"
	.include "include/icss_macros.inc"
	.include "rte_appl_defines.inc"
	.include "firmware_version.inc"

;define, set and asg section


;CCS specific settings
	.retain 	; Required for building .out with assembly file
	.retainrefs ; Required for building .out with assembly file
	.ref datalink_init
	.ref transport_init
	.ref load_code
	.global	main
	.global FN_RCV_TASK
	.global FN_EVENT_HANDLER
	.sect  ".text:main"

;********
;* MAIN *
;********
main:
	; PRU init
	; clear R0 to R29
	ZERO	&R0, 120

	; for initialising the IEP counters in local memory
	.if $defined(PRU0)
	ZERO	&TEMP_REG_1, 8
	SBCO	&TEMP_REG_1, ICSS_SHARED_RAM, IEP_CNT_HIGH_PRU0, 8
	; set IEP counter to 0
	SBCO	&TEMP_REG_1, ICSS_IEP, ICSS_IEP_COUNT_REG, 4
	; clear any pending IEP events
	LBCO	&TEMP_REG_1, ICSS_IEP, ICSS_IEP_CMP_STATUS_REG, 4
	SBCO	&TEMP_REG_1, ICSS_IEP, ICSS_IEP_CMP_STATUS_REG, 4
	; Todo: Do we need to clear IEP INTC event?
	; set CMP[0] event close to wrap-around
	LDI32	TEMP_REG_1, 0xFFFFFFFA
	SBCO	&TEMP_REG_1, ICSS_IEP, ICSS_IEP_CMP0_REG, 4
	; enable CMP[0] event
	LDI		TEMP_REG_1, 0x2
	SBCO	&TEMP_REG_1, ICSS_IEP, ICSS_IEP_CMP_CFG_REG, 4
	LDI		TEMP_REG_1, 0x551
	SBCO	&TEMP_REG_1, ICSS_IEP, ICSS_IEP_GLOBAL_CFG_REG, 4
	.else
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	.endif

	; for debug only: need to make sure that IEP_CNT_HIGH_PRU0 and IEP_CNT_HIGH_PRU1 have the same value
	LBCO	&TEMP_REG_1, ICSS_SHARED_RAM, IEP_CNT_HIGH_PRU0, 8
	QBEQ	DEBUG_SYNC_IEP_HIGH, TEMP_REG_1, TEMP_REG_2
	ZERO	&TEMP_REG_1, 8
	SBCO	&TEMP_REG_1, ICSS_SHARED_RAM, IEP_CNT_HIGH_PRU0, 8
DEBUG_SYNC_IEP_HIGH:

	;clear scratch pad banks
	XOUT	BANK0, &R0, 120
	XOUT	BANK1, &R0, 120
	XOUT	BANK2, &R0, 120

;init:
	; reset RX- and TX-FIFO and clear TX errors
	LDI		R31.w2, (D_RESET_RXFIFO | D_RX_ERROR_CLEAR | D_RESET_TXFIFO)
	; set R30 mask to take data from R30.w0 (TX path)
	LDI		R30.w2, 0xFFFF
	; generate a MDIO event to update port status in shared memory
	LDI32	TEMP_REG_1, 0x00200200	; Set MDIO_MII_LINK[1] and MDIO_MII_LINK[0] bit
	LDI		TEMP_REG_2.w0, ICSS_INTC_SRSR1
	SBCO	&TEMP_REG_1, ICSS_INTC, TEMP_REG_2.w0, 4

control_loop:
	; Receive task
	CALL	FN_RCV_TASK
	; event handler
	CALL	FN_EVENT_HANDLER
	QBA		control_loop

firmware_version:
	.if $defined(PRU0)
	.word 0x00		; PRU0
	.else
	.word 0x01		; PRU1
	.endif

	.word   ICSS_FIRMWARE_RELEASE_1
	.word   ICSS_FIRMWARE_RELEASE_2
