; Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
;
; Redistribution and use in source and binary forms, with or without
; modification, are permitted provided that the following conditions
; are met:
;
; Redistributions of source code must retain the above copyright
; notice, this list of conditions and the following disclaimer.
;
; notice, this list of conditions and the following disclaimer in the
; documentation and/or other materials provided with the
; distribution.
;
; Neither the name of Texas Instruments Incorporated nor the names of
; its contributors may be used to endorse or promote products derived
; from this software without specific prior written permission.
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
; LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
; A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
; OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
; SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
; LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
; DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
; (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
; OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;
; 	file:   mii_rcv.asm
;
; 	brief:
;
;	Version		Description								Author
; 	0.1     	Created									Thomas Mauer
;   0.2         RTE										Haisem Khan
;include section
	.include "include/macros.inc"
	.include "include/icss_regs.inc"
	.include "include/icss_macros.inc"
	.include "rte_appl_defines.inc"

	.global FN2_XFER_REG_DATA_TO_TX_FIFO
	.global FN_RCV_TASK

;*******************************************************************
; function receive task
;*******************************************************************
FN_RCV_TASK:
	; XFER in L2 buffer, R18.b0, byte counter
	XIN		RCV_L2_BANK0_ID, &R18.b0, 1
	; if rcv is not yet active, start with the receive first block (FB)
	QBBC	RCV_FB, RCV_STATUS_REG, RCV_ACTIVE_FLAG
	; when EOF flag set go to receive last blcok, otherwise to receive next block
	; NB will call LB if EOF flag is set by event handler
	QBA		RCV_NB
RCV_TASK_RETURN:
	RET

;*******************************************************************
; receive first block
;*******************************************************************
RCV_FB:
	; wait until L2 has received  RCV_LEVEL_START# of bytes
	QBGT	RCV_TASK_RETURN, R18.b0, RXL2_LEVEL_START
	; validate frame header
	XIN		RCV_L2_BANK0_ID, &R2, RANGE_R2_R13
	; is this SFD (0x5D)?
	; removed preamble check for short preamble case
	;QBNE	RCV_TASK_RETURN, RXL2_SFD_REGISTER, 0xD5
	; enable RCV_ACTIVE
	SET		RCV_STATUS_REG, RCV_STATUS_REG, RCV_ACTIVE_FLAG
	SET		RCV_STATUS_REG, RCV_STATUS_REG, RCV_HOST_PORT_FLAG

FB_SETUP_QUEUE:
	QBBC	FB_DONE, RCV_STATUS_REG, RCV_HOST_PORT_FLAG
	; get queue pointer
	ZERO	&MII_RCV, $sizeof(MII_RCV)
	; load queue descriptor and buffer offset
	.if $defined(PRU0)
	LDI		TEMP_REG_1.w0,  PORT2_RXPRU0_QUEUE_POINTER+2
	LBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, TEMP_REG_1.w0, 2	; get queue write poniter
	LDI		TEMP_REG_1.w2, PORT2_RXPRU0_BUFFER_ADDR
	.else
	LDI		TEMP_REG_1.w0,  PORT2_RXPRU1_QUEUE_POINTER+2
	LBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, TEMP_REG_1.w0, 2 	; get queue write poniter
	LDI		TEMP_REG_1.w2, PORT2_RXPRU1_BUFFER_ADDR
	.endif
	; calculate buffer offset
	LSL		TEMP_REG_1.w0, TEMP_REG_1.w0, 5
	ADD		MII_RCV.host_buffer_ptr, TEMP_REG_1.w0, TEMP_REG_1.w2

	CALL1	FN1_CALC_FREE_QUEUE_SPACE
	QBNE	FB_SETUP_HEADER_SPACE, TEMP_REG_2.b0, 0
	CLR		RCV_STATUS_REG, RCV_STATUS_REG, RCV_HOST_PORT_FLAG
	QBA		RCV_TASK_RETURN

FB_SETUP_HEADER_SPACE:
	; initialize header and timestamp space to all 0's
	ZERO	&R19, RANGE_R19_R26
	SBCO	&R19.b0, OCMC_RAM, MII_RCV.host_buffer_ptr, 32

;FB_STORE_TIMESTAMP:
	.if $defined(PRU0)
	; Load IEP Timer Value
	LBCO	&TEMP_REG_3, ICSS_IEP, ICSS_IEP_CAPR0_REG, 4			; Port 0
	; Load TS_HIGH from shared memory
	LBCO	&TEMP_REG_1, ICSS_SHARED_RAM, IEP_CNT_HIGH_PRU0, 4
	; Check if TS_HIGH_UPDATE_FLAG is set, If TS_HIGH_UPDATE_FLAG is not set
	; take the value of TS_HIGH as it is from shared memory
	QBBS	FB_TS_HIGH_UPDATE_BEFORE_FB, RCV_STATUS_REG, RCV_TS_HIGH_UPDATE_FLAG
	; Store upper part of TS
	ADD		TEMP_REG_2, MII_RCV.host_buffer_ptr, IEP_HIGH_COUNTER_OFFSET
	SBCO	&TEMP_REG_1, OCMC_RAM, TEMP_REG_2, 4
	QBA		FB_STORE_TS_LOW
	; If TS_HIGH_UPDATE_FLAG is set check if the frame arrived before the wrap-around of IEP Timer
FB_TS_HIGH_UPDATE_BEFORE_FB:
	; Check if IEP Timer value is under 1us
	; Loading TEMP_REG_4 with value of 1us
	LDI 	TEMP_REG_4, 1000
	QBLE	FB_TS_LOW_LESS_THAN_1US, TEMP_REG_4, TEMP_REG_3
	; If IEP Timer value is above 1us and TS_HIGH_UPDATE_FLAG is set this means frame arrived
	; just before the IEP Timer wrap-around so we need to deduct one from IEP High portion
	SUB		TEMP_REG_1, TEMP_REG_1, 1
	; Store upper part of TS
	ADD		TEMP_REG_2, MII_RCV.host_buffer_ptr, IEP_HIGH_COUNTER_OFFSET
	SBCO	&TEMP_REG_1, OCMC_RAM, TEMP_REG_2, 4
	QBA		FB_STORE_TS_LOW
FB_TS_LOW_LESS_THAN_1US:
	; If IEP Timer Value is less than 1us and TS_HIGH_UPDATE_FLAG is set this means frame arrived
	; after the IEP Timer wrap-around so we take the value of TS_HIGH as it is from shared memory
	; Store upper part of TS
	ADD		TEMP_REG_2, MII_RCV.host_buffer_ptr, IEP_HIGH_COUNTER_OFFSET
	SBCO	&TEMP_REG_1, OCMC_RAM, TEMP_REG_2, 4
	.else
	; Load IEP Timer Value
	LBCO	&TEMP_REG_3, ICSS_IEP, ICSS_IEP_CAPR2_REG, 4			; Port 1
	; Load TS_HIGH from shared memory
	LBCO	&TEMP_REG_1, ICSS_SHARED_RAM, IEP_CNT_HIGH_PRU1, 4
	; Check if TS_HIGH_UPDATE_FLAG is set, If TS_HIGH_UPDATE_FLAG is not set
	; take the value of TS_HIGH as it is from shared memory
	QBBS	FB_TS_HIGH_UPDATE_BEFORE_FB, RCV_STATUS_REG, RCV_TS_HIGH_UPDATE_FLAG
	; Store upper part of TS
	ADD		TEMP_REG_2, MII_RCV.host_buffer_ptr, IEP_HIGH_COUNTER_OFFSET
	SBCO	&TEMP_REG_1, OCMC_RAM, TEMP_REG_2, 4
	QBA		FB_STORE_TS_LOW
	; If TS_HIGH_UPDATE_FLAG is set check if the frame arrived before the wrap-around of IEP Timer
FB_TS_HIGH_UPDATE_BEFORE_FB:
	; Check if IEP Timer value is under 1us
	; Loading TEMP_REG_4 with value of 1us
	LDI 	TEMP_REG_4, 1000
	QBLE	FB_TS_LOW_LESS_THAN_1US, TEMP_REG_4, TEMP_REG_3
	; If IEP Timer value is above 1us and TS_HIGH_UPDATE_FLAG is set this means frame arrived
	; just before the IEP Timer wrap-around so we need to deduct one from IEP High portion
	SUB		TEMP_REG_1, TEMP_REG_1, 1
	; Store upper part of TS
	ADD		TEMP_REG_2, MII_RCV.host_buffer_ptr, IEP_HIGH_COUNTER_OFFSET
	SBCO	&TEMP_REG_1, OCMC_RAM, TEMP_REG_2, 4
	QBA		FB_STORE_TS_LOW
FB_TS_LOW_LESS_THAN_1US:
	; If IEP Timer Value is less than 1us and TS_HIGH_UPDATE_FLAG is set this means frame arrived
	; after the IEP Timer wrap-around so we take the value of TS_HIGH as it is from shared memory
	; Store upper part of TS
	ADD		TEMP_REG_2, MII_RCV.host_buffer_ptr, IEP_HIGH_COUNTER_OFFSET
	SBCO	&TEMP_REG_1, OCMC_RAM, TEMP_REG_2, 4
	.endif
FB_STORE_TS_LOW:
	; Store lower part of TS
	ADD		TEMP_REG_2, MII_RCV.host_buffer_ptr, IEP_LOW_COUNTER_OFFSET
	SBCO	&TEMP_REG_3, OCMC_RAM, TEMP_REG_2, 4

;FB_INCREMENT_BUFFER_POINTER:
	; increment buffer pointer and check for buffer wrap-around
	ADD		MII_RCV.host_buffer_ptr, MII_RCV.host_buffer_ptr, 32
	.if $defined(PRU0)
	LDI		TEMP_REG_1.w0, PORT2_RXPRU1_BUFFER_ADDR
	QBGT	FB_DONE, MII_RCV.host_buffer_ptr, TEMP_REG_1.w0
	LDI		MII_RCV.host_buffer_ptr, PORT2_RXPRU0_BUFFER_ADDR
	.else
	LDI		TEMP_REG_1.w0, PORT2_RXPRU1_BUFFER_ADDR_END
	QBGT	FB_DONE, MII_RCV.host_buffer_ptr, TEMP_REG_1.w0
	LDI		MII_RCV.host_buffer_ptr, PORT2_RXPRU1_BUFFER_ADDR
	.endif

FB_DONE:
	LDI		R0.b0, SHIFT_MII_RCV_PARAMS
	.if $defined(PRU0)
	XOUT	BANK1, &MII_RCV, $sizeof(MII_RCV)
	.else
	XOUT	BANK2, &MII_RCV, $sizeof(MII_RCV)
	.endif
	QBA		RCV_TASK_RETURN


;*******************************************************************
; receive next block
;*******************************************************************
RCV_NB:
	QBBC	RCV_NB_RETURN, RCV_STATUS_REG, RCV_HOST_PORT_FLAG
	; FW checks if more then 32 bytes received in this bank?
	; shift bit RCV_L2_CURRENT_WORKING_BANK to bit 5, so FW can compare it to bit 5 of R18.b0
	LSL		TEMP_REG_1.b0, RCV_STATUS_REG, (5-RCV_L2_CURRENT_WORKING_BANK)
	XOR		TEMP_REG_1.b0, TEMP_REG_1.b0, R18.b0
	; if bit is set, then FW can process 32 bytes
	QBBC	RCV_NB_RETURN, TEMP_REG_1.b0, 5
	; get receive parameters from scratch pad
	LDI		R0.b0, SHIFT_MII_RCV_PARAMS
	.if $defined(PRU0)
	XIN		BANK1, &MII_RCV, $sizeof(MII_RCV)
	.else
	XIN		BANK2, &MII_RCV, $sizeof(MII_RCV)
	.endif
	; increment byte counter
	ADD		MII_RCV.rcv_byte_cnt, MII_RCV.rcv_byte_cnt, 32
	; get RXL2 host working bank
	XIN		RCV_L2_BANK0_ID, &R2, RANGE_R2_R13
	QBBC	RCV_NB_HOST_CALC_FREE_SPACE, RCV_STATUS_REG, RCV_L2_CURRENT_WORKING_BANK
	XIN		RCV_L2_BANK1_ID, &R2, RANGE_R2_R13

RCV_NB_HOST_CALC_FREE_SPACE:
	CALL1	FN1_CALC_FREE_QUEUE_SPACE
	QBNE	RCV_NB_HOST_QUEUE_STORE_FRAME, TEMP_REG_2.b0, 0
	CLR		RCV_STATUS_REG, RCV_STATUS_REG, RCV_HOST_PORT_FLAG
	QBA		RCV_NB_FWD_CALC_FREE_SPACE
RCV_NB_HOST_QUEUE_STORE_FRAME:
	;store frame
	SBCO	&R2.b0, OCMC_RAM, MII_RCV.host_buffer_ptr, 32
	; increment buffer pointer and check for buffer wrap-around
	ADD		MII_RCV.host_buffer_ptr, MII_RCV.host_buffer_ptr, 32
	.if $defined(PRU0)
	LDI		TEMP_REG_1.w0, PORT2_RXPRU1_BUFFER_ADDR
	QBGT	RCV_NB_FWD_CALC_FREE_SPACE, MII_RCV.host_buffer_ptr, TEMP_REG_1.w0
	LDI		MII_RCV.host_buffer_ptr, PORT2_RXPRU0_BUFFER_ADDR
	.else
	LDI		TEMP_REG_1.w0, PORT2_RXPRU1_BUFFER_ADDR_END
	QBGT	RCV_NB_FWD_CALC_FREE_SPACE, MII_RCV.host_buffer_ptr, TEMP_REG_1.w0
	LDI		MII_RCV.host_buffer_ptr, PORT2_RXPRU1_BUFFER_ADDR
	.endif
RCV_NB_FWD_CALC_FREE_SPACE:
	; store receive parameters to scratch pad.
	LDI		R0.b0, SHIFT_MII_RCV_PARAMS
	.if $defined(PRU0)
	XOUT	BANK1, &MII_RCV, $sizeof(MII_RCV)
	.else
	XOUT	BANK2, &MII_RCV, $sizeof(MII_RCV)
	.endif
	; switch bank token
	XOR		RCV_STATUS_REG, RCV_STATUS_REG, 1<<RCV_L2_CURRENT_WORKING_BANK

RCV_NB_RETURN:
	QBBS	RCV_LB, RCV_STATUS_REG, RCV_EOF_EVENT_FLAG
	QBA		RCV_TASK_RETURN

;*******************************************************************
; receive last block
;*******************************************************************
RCV_LB:
	QBBC	RCV_LB_RETURN, RCV_STATUS_REG, RCV_HOST_PORT_FLAG
	; get receive parameters from scratch pad
	LDI		R0.b0, SHIFT_MII_RCV_PARAMS
	.if $defined(PRU0)
	XIN		BANK1, &MII_RCV, $sizeof(MII_RCV)
	.else
	XIN		BANK2, &MII_RCV, $sizeof(MII_RCV)
	.endif
;when bank wrap-around occurs, handle first the 32-bytes, then the remainer
	XIN		RCV_L2_BANK0_ID, &R2, RANGE_R2_R13
	QBBC	RCV_LB_L2_FREE_QUEUE_SPACE, RCV_STATUS_REG, RCV_L2_CURRENT_WORKING_BANK
	XIN		RCV_L2_BANK1_ID, &R2, RANGE_R2_R13
RCV_LB_L2_FREE_QUEUE_SPACE:
	; R18.b0 range check (if R18.b0>=32 --> substract 32 by clearing bit 5)
	CLR		REMAINING_BYTES_CNT, R18.b0, 5
	.if $defined(PRU0)
	LDI		TEMP_REG_1.w0, PORT2_RXPRU0_QUEUE_POINTER
	LBCO	&TEMP_REG_1, ICSS_SHARED_RAM, TEMP_REG_1.w0, 4
	.else
	LDI		TEMP_REG_1.w0, PORT2_RXPRU1_QUEUE_POINTER
	LBCO	&TEMP_REG_1, ICSS_SHARED_RAM, TEMP_REG_1.w0, 4
	.endif
	CALL1	FN1_CALC_FREE_QUEUE_SPACE
	QBNE	RCV_LB_STORE_FRAME, TEMP_REG_2.b0, 0
	CLR		RCV_STATUS_REG, RCV_STATUS_REG, RCV_HOST_PORT_FLAG
	QBA		RCV_LB_RETURN
RCV_LB_STORE_FRAME:
	;store frame
	SBCO	&R2.b0, OCMC_RAM, MII_RCV.host_buffer_ptr, b3	; => R0.b3 = REMAINING_BYTES_CNT
	; store receive parameters to scratch pad.
	ADD		MII_RCV.rcv_byte_cnt, MII_RCV.rcv_byte_cnt, REMAINING_BYTES_CNT
	; remove CRC from frame length
	;SUB		MII_RCV.rcv_byte_cnt, MII_RCV.rcv_byte_cnt, CRC_LENGTH
	; extract frame status info
	SUB		R1.b3, R18.b0, 1		;set L2 write pointer back to last entry
	AND		R1.b3, R1.b3, 0x1F		; adjust wrap-around and >32bytes (Modulo 32)
	LSR		R1.b3, R1.b3, 1			; 2 bytes of frame data is combines into 1 common byte status info.
	ADD		R1.b3, R1.b3, (10*4)	; byte offset to R10
	MVIB	TEMP_REG_1.b2, *R1.b3
	LDI32	TEMP_REG_4, 0
	AND		TEMP_REG_4.b2, TEMP_REG_1.b2, 0xC1	; mask error bits in RXL2 status
	QBEQ	RCV_LB_STORE_HEADER_INFO, TEMP_REG_4.b2, 0
	QBBC	NO_CRC_ERROR, TEMP_REG_4.b2, ERROR_CRC_BIT
	SET		TEMP_REG_4.b0, TEMP_REG_4.b0, CRC_ERROR_ESL_BIT
NO_CRC_ERROR:
	QBBC	NO_ALIGNMENT_ERROR, TEMP_REG_4.b2, ERROR_NIBBLE_BIT
	SET		TEMP_REG_4.b0, TEMP_REG_4.b0, ALIGNMENT_ERROR_ESL_BIT
NO_ALIGNMENT_ERROR:
	QBBC	NO_RX_ERROR, TEMP_REG_4.b2, RX_ERR_BIT
	SET		TEMP_REG_4.b0, TEMP_REG_4.b0, RX_ERR_ESL_BIT
NO_RX_ERROR:
	; update statistic because of error
	.if $defined(PRU0)
	LBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, STAT_CRCERR_P0, 2
	ADD		TEMP_REG_1.w0, TEMP_REG_1.w0, 1
	SBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, STAT_CRCERR_P0, 2
	.else
	LBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, STAT_CRCERR_P1, 2
	ADD		TEMP_REG_1.w0, TEMP_REG_1.w0, 1
	SBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, STAT_CRCERR_P1, 2
	.endif
RCV_LB_STORE_HEADER_INFO:
	; load original pointer from queue descriptor and buffer offset
	.if $defined(PRU0)
	LDI		TEMP_REG_1.w0,  PORT2_RXPRU0_QUEUE_POINTER+2
	LBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, TEMP_REG_1.w0, 2	; get queue write poniter
	LDI		TEMP_REG_1.w2, PORT2_RXPRU0_BUFFER_ADDR
	.else
	LDI		TEMP_REG_1.w0,  PORT2_RXPRU1_QUEUE_POINTER+2
	LBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, TEMP_REG_1.w0, 2 ; get queue write poniter
	LDI		TEMP_REG_1.w2, PORT2_RXPRU1_BUFFER_ADDR
	.endif
	; calculate header offset in buffer in OCMC
	LSL		TEMP_REG_1.w0, TEMP_REG_1.w0, 5
	ADD		TEMP_REG_1.w0, TEMP_REG_1.w0, TEMP_REG_1.w2
	ADD		TEMP_REG_1.w0, TEMP_REG_1.w0, PACKET_DESCRIPTOR_OFFSET
	; set the header field
	LDI		TEMP_REG_2, 0
	LSL		TEMP_REG_2, MII_RCV.rcv_byte_cnt, 16
	SET		TEMP_REG_2, TEMP_REG_2, TIMESTAMP_BIT
	.if $defined(PRU0)
	CLR		TEMP_REG_2, TEMP_REG_2, PORT_BIT
	.else
	SET		TEMP_REG_2, TEMP_REG_2, PORT_BIT
	.endif
	SBCO	&TEMP_REG_2, OCMC_RAM, TEMP_REG_1.w0, 4
RCV_LB_NO_FRAME_ERROR:
	; update queue desc and buffer desc in shared memory
	.if $defined(PRU0)
	; load queue desc
	LDI		TEMP_REG_1.w2, PORT2_RXPRU0_QUEUE_POINTER+2
	LBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, TEMP_REG_1.w2, 2	; get original write pointer
	; calculated buffer desc
	LSL		TEMP_REG_2.w0, TEMP_REG_1.w0, 2						; Offset calculation for buffer desc based on write pointer
	LDI		TEMP_REG_2.w2, PORT2_RXPRU0_BUFFER_DESC				; buffer desc base addr
	ADD		TEMP_REG_2.w0, TEMP_REG_2.w0, TEMP_REG_2.w2			; buffer desc base addr + offset
	; preapre buffer descr content
	LSL		TEMP_REG_3, MII_RCV.rcv_byte_cnt, 16				; Interested in storing the byte count or packet length
	ADD		TEMP_REG_3.b0, TEMP_REG_3.b0, TEMP_REG_4.b0			; Add the error bits information according to the ESL format and store it in shared memory
STORE_FRAME_STATUS_INFO:
	SBCO	&TEMP_REG_3, ICSS_SHARED_RAM, TEMP_REG_2.w0, 4		; storing packet length information in the buffer desc
	; update queue desc
	ADD		TEMP_REG_2.w0, MII_RCV.rcv_byte_cnt, 31				; Integer division (x + 31) / 32 to calculate the new write pointer
	LSR		TEMP_REG_2.w0, TEMP_REG_2.w0, 5						; which points to the next empty place
	ADD		TEMP_REG_1.w0, TEMP_REG_2.w0, TEMP_REG_1.w0
	ADD		TEMP_REG_1.w0, TEMP_REG_1.w0, 1						; Add 1 block of 32 bytes because of the Header and Timestamp Block
	LDI		TEMP_REG_3.w0, BUFFER_DESC_COUNT
	QBGT	RCV_LB_HOST_QUEUE_DESC, TEMP_REG_1.w0, TEMP_REG_3.w0
	SUB		TEMP_REG_1.w0, TEMP_REG_1.w0, TEMP_REG_3.w0			; Handling wrap around for the write pointer
RCV_LB_HOST_QUEUE_DESC:
	SBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, TEMP_REG_1.w2, 2	; Storing the new write pointer value in shared memory
	.else
	; load queue desc
	LDI		TEMP_REG_1.w2, PORT2_RXPRU1_QUEUE_POINTER+2
	LBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, TEMP_REG_1.w2, 2
	; calculated buffer desc
	LSL		TEMP_REG_2.w0, TEMP_REG_1.w0, 2
	LDI		TEMP_REG_2.w2, PORT2_RXPRU1_BUFFER_DESC
	ADD		TEMP_REG_2.w0, TEMP_REG_2.w0, TEMP_REG_2.w2
	; preapre buffer descr content
	LSL		TEMP_REG_3, MII_RCV.rcv_byte_cnt, 16
	ADD		TEMP_REG_3.b0, TEMP_REG_3.b0, TEMP_REG_4.b0			; Add the error bits information according to the ESL format and store it in shared memory
STORE_FRAME_STATUS_INFO:
	SBCO	&TEMP_REG_3, ICSS_SHARED_RAM, TEMP_REG_2.w0, 4
	; update queue desc
	ADD		TEMP_REG_2.w0, MII_RCV.rcv_byte_cnt, 31
	LSR		TEMP_REG_2.w0, TEMP_REG_2.w0, 5
	ADD		TEMP_REG_1.w0, TEMP_REG_2.w0, TEMP_REG_1.w0
	ADD		TEMP_REG_1.w0, TEMP_REG_1.w0, 1						; Add 1 block of 32 bytes because of the Header and Timestamp Block
	LDI		TEMP_REG_3.w0, BUFFER_DESC_COUNT
	QBGT	RCV_LB_HOST_QUEUE_DESC, TEMP_REG_1.w0, TEMP_REG_3.w0
	SUB		TEMP_REG_1.w0, TEMP_REG_1.w0, TEMP_REG_3.w0
RCV_LB_HOST_QUEUE_DESC:
	SBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, TEMP_REG_1.w2, 2
	.endif
		; update statistic
	.if $defined(PRU0)
	LBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, STAT_RCV_P0, 2
	ADD		TEMP_REG_1.w0, TEMP_REG_1.w0, 1
	SBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, STAT_RCV_P0, 2
	.else
	LBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, STAT_RCV_P1, 2
	ADD		TEMP_REG_1.w0, TEMP_REG_1.w0, 1
	SBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, STAT_RCV_P1, 2
	.endif
RCV_LB_RETURN:
	;reset RX FIFO
	LDI		R31.w2, (D_RESET_RXFIFO | D_RX_ERROR_CLEAR)
	; Clear all flags in RCV_STATUS_REG
	LDI		RCV_STATUS_REG, 0
	QBA		RCV_TASK_RETURN


;*******************************************************************
; calculates free queue space
;*******************************************************************
; Call only with CALL1
FN1_CALC_FREE_QUEUE_SPACE:
	LDI		TEMP_REG_3.w0, BUFFER_DESC_COUNT
	.if $defined(PRU0)
	; get queue read poniter
	LDI		TEMP_REG_1.w2, PORT2_RXPRU0_QUEUE_POINTER
	LBCO	&TEMP_REG_1.w2, ICSS_SHARED_RAM, TEMP_REG_1.w2, 2
	; get queue write poniter
	LSR		TEMP_REG_1.w0, MII_RCV.host_buffer_ptr, 5				; added 11-7-2016
	NOP
	NOP
	.else
	; get queue read poniter
	LDI		TEMP_REG_1.w2, PORT2_RXPRU1_QUEUE_POINTER
	LBCO	&TEMP_REG_1.w2, ICSS_SHARED_RAM, TEMP_REG_1.w2, 2
	; get queue write poniter
	LSR		TEMP_REG_1.w0, MII_RCV.host_buffer_ptr, 5				; added 11-7-2016
	LDI		TEMP_REG_2.w0, BUFFER_DESC_COUNT
	SUB		TEMP_REG_1.w0, TEMP_REG_1.w0, TEMP_REG_2.w0
	.endif

	QBGT	READ_GREATER_THAN_WRITE, TEMP_REG_1.w0, TEMP_REG_1.w2	; if rd /> wr
	SUB		TEMP_REG_4.w0, TEMP_REG_3.w0, TEMP_REG_1.w0				; then size - wr
	ADD		TEMP_REG_4.w0, TEMP_REG_4.w0, TEMP_REG_1.w2				; rd + (size - wr)
	QBLT	SPACE_AVAILABLE, TEMP_REG_4.w0, 1 						; if rd + (size - wr) > 1 then space available otherwise not
	LDI		TEMP_REG_2.b0, 0										; No, space is not available
	QBA		SPACE_NOT_AVAILABLE
SPACE_AVAILABLE:
	LDI		TEMP_REG_2.b0, 1										; Yes, space is available
	QBA	 	CFQS_RETURN
READ_GREATER_THAN_WRITE:
	SUB		TEMP_REG_4.w0, TEMP_REG_1.w2, TEMP_REG_1.w0				; if rd > wr then rd - wr
	QBLT	SPACE_AVAILABLE, TEMP_REG_4.w0, 1 						; if rd - wr > 1 then space available otherwise not
	LDI		TEMP_REG_2.b0, 0										; No, space is not available
SPACE_NOT_AVAILABLE:
	; update statistic for Overflow
	.if $defined(PRU0)
	LBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, STAT_RCV_OVERFLOW_P0, 2
	ADD		TEMP_REG_1.w0, TEMP_REG_1.w0, 1
	SBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, STAT_RCV_OVERFLOW_P0, 2
	.else
	LBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, STAT_RCV_OVERFLOW_P1, 2
	ADD		TEMP_REG_1.w0, TEMP_REG_1.w0, 1
	SBCO	&TEMP_REG_1.w0, ICSS_SHARED_RAM, STAT_RCV_OVERFLOW_P1, 2
	.endif
CFQS_RETURN:
	RET1
