/*
 * Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <xdc/std.h>
#include <xdc/runtime/Error.h>
#include <xdc/runtime/System.h>

#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/hal/Core.h>
#include <ti/sysbios/family/arm/a8/Mmu.h>

#include <ti/ndk/inc/stkmain.h>

#include <ti/drv/emac/emac_drv.h>
#include <ti/drv/emac/src/v4/emac_drv_v4.h>
#include <ti/drv/pruss/soc/am437x/pruicss_device.c>

#include <ti/starterware/include/types.h>
#include <ti/starterware/include/hw/hw_types.h>
#include <ti/starterware/include/hw/hw_control_am43xx.h>
#include <ti/starterware/include/hw/am437x.h>
#include <ti/starterware/include/ethernet.h>

//=======================
// Added
#include "board.h"
#include "console_utils.h"

#include <ti/starterware/include/hw/hw_icss.h>
#include <ti/starterware/include/hw/hw_icss_cfg.h>
#include <ti/starterware/include/hw/hw_icss_pru_ctrl.h>
#include <ti/starterware/include/hw/hw_icss_mii_rt_cfg.h>
#include <ti/starterware/include/hw/hw_icss_mii_mdio.h>
#include <ti/starterware/include/hw/hw_icss_iep.h>
#include "rte_appl_def.h"
#include "rte_drv.h"

#include <ti/drv/pruss/pruicss.h>

#include "ti_pruss_intc_mapping.h"
//=======================

#include <ti/board/board.h>

/* UART Header files */
#include <ti/drv/uart/UART.h>
#include <ti/drv/uart/UART_stdio.h>

/* GPIO header files and defines*/
#include <ti/drv/gpio/GPIO.h>
#include <ti/drv/gpio/soc/GPIO_v1.h>

// PRU RTE Tracer header files
#include "RTE_TRACER_PRU_ICSS_PRU0.h"
#include "RTE_TRACER_PRU_ICSS_PRU1.h"

#define GPIO_USER0_LED_PIN_NUM    (0x0B)
#define GPIO_USER0_LED_PORT_NUM   (0x05)
#define GPIO_USER1_LED_PIN_NUM    (0x0A)
#define GPIO_USER1_LED_PORT_NUM   (0x05)
/* Enable the below macro to have prints on the IO Console */
//#define IO_CONSOLE

#ifndef IO_CONSOLE
#define NIMU_log                UART_printf
#else
#define NIMU_log                printf
#endif

#define IND_PKG_VERSION "Real-Time Ethernet Tracer 1.0.0.0"
//#define PRU0_FIRMWARE_NAME      PRU0_FIRMWARE        // name of the C struct in PRU header file
//#define PRU1_FIRMWARE_NAME      PRU1_FIRMWARE        // name of the C struct in PRU header file
/* ========================================================================== */
/*                             Macros                                 		  */
/* ========================================================================== */

#define MAX_TABLE_ENTRIES   3
#define PRUICSS_INSTANCE 2

#define PRUSS1_MII_RT						0x32000
#define PRUSS_MII_RT_RXCFG0					0x0
#define PRUSS_MII_RT_RXCFG1					0x4
#define PRUSS_MII_RT_TXCFG0					0x10
#define PRUSS_MII_RT_TXCFG1					0x14

// main port descriptors - local for now
t_portParams switchPort[3];						// fixed set of 2 ports
#define PORT0 0									// Port 0: PRU0-RX
#define PORT1 1									// Port 1: PRU1_RX
#define PORT2 2									// Port 2: PC

Uint8	ARM_buffer[1600] = {0};

//Uint8 ET2000_SYMBOLIC_MAC_ADDR[6] = {0x01, 0x01, 0x05, 0x10, 0x00, 0x00};
//In the following, first 6 bytes for Symbolic mac 01 01 05 10 00 00, 7th byte for port number and 8th byte for error information, we manipulate only 7th and 8th byte always
Uint8 ET2000_Header_Upper_Part_Array[8] = {0x01, 0x01, 0x05, 0x10, 0x00, 0x00, 0x00, 0x00};

Uint32	timestamp_store_array_port0[2] = {0};
Uint32	timestamp_store_array_port1[2] = {0};

#define ARM_BUFFER_ARRAY_ROWS				20
#define ARM_BUFFER_ARRAY_COLUMNS			1600

#define PRUSS_SHAREDRAM_BASE				0x54410000
#define L3_OCMC_BASE_ADDRESS				0x40300000

// read/write pointer of port queues
#define PORT2_RXPRU0_QUEUE_POINTER			0x0100	// handled by PRU0
#define PORT2_RXPRU1_QUEUE_POINTER			0x0104	// handled by PRU1

// buffer descriptor base address: 32 buffer descriptors per queue
//#define	BUFFER_DESC_COUNT					200
#define	PORT2_RXPRU0_BUFFER_DESC			0x0120
#define	PORT2_RXPRU1_BUFFER_DESC			(PORT2_RXPRU0_BUFFER_DESC + BUFFER_DESC_COUNT*4)
#define	PORT2_RXPRU1_BUFFER_DESC_END		(PORT2_RXPRU1_BUFFER_DESC + BUFFER_DESC_COUNT*4)
#define BUFFER_DESC_FRAME_ERROR_FLAG		0

// frame queue base address in OCMC/L3 memory
#define	BUFFER_SIZE							32					//size of each queue entry in bytes
#define	QUEUE_COUNT							BUFFER_DESC_COUNT	//number of queue entries
#define	TOTAL_BUFFER_SIZE					(BUFFER_SIZE * QUEUE_COUNT)
#define	PORT2_RXPRU0_BUFFER_ADDR			0x0000
#define	PORT2_RXPRU1_BUFFER_ADDR			(PORT2_RXPRU0_BUFFER_ADDR +  BUFFER_SIZE*QUEUE_COUNT)
#define	PORT2_RXPRU1_BUFFER_ADDR_END		(PORT2_RXPRU1_BUFFER_ADDR +  BUFFER_SIZE*QUEUE_COUNT)

// IEP counters and Header offset inside packet block in OCMC
#define	IEP_LOW_COUNTER_OFFSET				0x4			// .field32
#define	IEP_HIGH_COUNTER_OFFSET				0x8			// .field32
#define	HEADER_OFFSET						0x0			// .field32

// Overflow Stats in PRU's Shared Memory
#define STAT_RCV_OVERFLOW_P0				0x002C		// .field16
#define STAT_RCV_OVERFLOW_P1				0x002E		// .field16

// Diagnostics Arrays
#define DIAGNOSTIC_ARRAYS_SIZE				3000
int empty_space_diagnostic_array_pru0[DIAGNOSTIC_ARRAYS_SIZE] = {0};
int empty_space_diagnostic_array_pru1[DIAGNOSTIC_ARRAYS_SIZE] = {0};
int overflow_diagnostic_array_pru0[DIAGNOSTIC_ARRAYS_SIZE] = {0};
int overflow_diagnostic_array_pru1[DIAGNOSTIC_ARRAYS_SIZE] = {0};

#define	bufSize								1000
/* ========================================================================== */
/*                            Global Variables                                */
/* ========================================================================== */

/**Task handle for EIP*/
Task_Handle main_task;

static int nimu_device_index = 0U;
t_mdio_params mdio_configuration = {0x9F, 0x0, 0x1, 0, 0, 0};	// default configuration for ICEv2

NIMU_DEVICE_TABLE_ENTRY NIMUDeviceTable[MAX_TABLE_ENTRIES];

void TaskFxn(UArg a0, UArg a1);
void TaskDiagnostics(UArg a0, UArg a1);

int MDIO_Init(t_mdio_params *mdio_conf);
void PRUSSDRVPruInit(void);
int portInit(void);
Int32 RxPktInfo(void);
Int32 RxPktTransfer_ESL(void);
Int32 Copy_Packet_PRU2ARM_ESL (Uint8 port_number); //Returns Packet Length
int Diagnostics_FreeSpace(Uint16 rd_pos, Uint16 wr_pos);

extern int CpswEmacInit (STKEVENT_Handle hEvent);
/* GPIO Driver board specific pin configuration structure */
GPIO_PinConfig gpioPinConfigs[] = {
    /* Input pin with interrupt enabled */
    GPIO_DEVICE_CONFIG(( GPIO_USER0_LED_PORT_NUM + 1 ), GPIO_USER0_LED_PIN_NUM) |
    GPIO_CFG_IN_INT_RISING | GPIO_CFG_INPUT,

    /* Output pin */
    GPIO_DEVICE_CONFIG( (GPIO_USER0_LED_PORT_NUM + 1), GPIO_USER0_LED_PIN_NUM) |
    GPIO_CFG_OUTPUT
};

/* GPIO Driver call back functions */
GPIO_CallbackFxn gpioCallbackFunctions[] = {
    NULL,
    NULL
};

/* GPIO Driver configuration structure */
GPIO_v1_Config GPIO_v1_config = {
    gpioPinConfigs,
    gpioCallbackFunctions,
    sizeof(gpioPinConfigs) / sizeof(GPIO_PinConfig),
    sizeof(gpioCallbackFunctions) / sizeof(GPIO_CallbackFxn),
    0,
    };

PRUICSS_Handle pruIcss1Handle;
/* ========================================================================== */
/*                          Function Definitions                              */
/* ========================================================================== */

void CpswPortMacModeSelect(uint32_t portNum, uint32_t macMode)
{
    uint32_t regVal = 0U;

    regVal = HW_RD_REG32(SOC_CONTROL_MODULE_REG + CTRL_GMII_SEL);

    switch(macMode)
    {
        case ETHERNET_MAC_TYPE_MII:
        case ETHERNET_MAC_TYPE_GMII:
            if(1U == portNum)
            {
                HW_SET_FIELD(regVal, CTRL_GMII_SEL_GMII1, 0U);
                HW_SET_FIELD(regVal, CTRL_GMII_SEL_RGMII1_IDMODE, 0U);
                HW_SET_FIELD(regVal, CTRL_GMII_SEL_RMII1_IO_CLK_EN, 0U);
            }
            else if(2U == portNum)
            {
                HW_SET_FIELD(regVal, CTRL_GMII_SEL_GMII2, 0U);
                HW_SET_FIELD(regVal, CTRL_GMII_SEL_RGMII2_IDMODE, 0U);
                HW_SET_FIELD(regVal, CTRL_GMII_SEL_RMII2_IO_CLK_EN, 0U);
            }
            else
            {
                /* This error does not happen because of check done already */
            }
            break;

        case ETHERNET_MAC_TYPE_RMII: /* RMII */
            if(1U == portNum)
            {
                HW_SET_FIELD(regVal, CTRL_GMII_SEL_GMII1, 1U);
                HW_SET_FIELD(regVal, CTRL_GMII_SEL_RGMII1_IDMODE, 0U);
                HW_SET_FIELD(regVal, CTRL_GMII_SEL_RMII1_IO_CLK_EN, 0U);
            }
            else if(2U == portNum)
            {
                HW_SET_FIELD(regVal, CTRL_GMII_SEL_GMII2, 1U);
                HW_SET_FIELD(regVal, CTRL_GMII_SEL_RGMII2_IDMODE, 0U);
                HW_SET_FIELD(regVal, CTRL_GMII_SEL_RMII2_IO_CLK_EN, 0U);
            }
            else
            {
                /* This error does not happen because of check done already */
            }
            break;

        case ETHERNET_MAC_TYPE_RGMII: /* RGMII */
            if(1U == portNum)
            {
                HW_SET_FIELD(regVal, CTRL_GMII_SEL_GMII1, 2U);
                HW_SET_FIELD(regVal, CTRL_GMII_SEL_RGMII1_IDMODE, 0U);
                HW_SET_FIELD(regVal, CTRL_GMII_SEL_RMII1_IO_CLK_EN, 0U);
            }
            else if(2U == portNum)
            {
                HW_SET_FIELD(regVal, CTRL_GMII_SEL_GMII2, 2U);
                HW_SET_FIELD(regVal, CTRL_GMII_SEL_RGMII2_IDMODE, 0U);
                HW_SET_FIELD(regVal, CTRL_GMII_SEL_RMII2_IO_CLK_EN, 0U);
            }
            else
            {
                /* This error does not happen because of check done already */
            }
            break;

         default:
         break;
    }

    HW_WR_REG32((SOC_CONTROL_MODULE_REG + CTRL_GMII_SEL), regVal);
}

/**
 * \brief   This function returns the MAC address for the EVM
 *
 * \param   addrIdx    the MAC address index.
 * \param   macAddr    the Pointer where the MAC address shall be stored
 *     'addrIdx' can be either 0 or 1
 *
 * \return  None.
 */
void EVMMACAddrGet(uint32_t addrIdx, uint8_t *macAddr)
{
    switch(addrIdx)
    {
        case 1U:
            macAddr[0U] = HW_RD_FIELD32(SOC_CONTROL_MODULE_REG + CTRL_MAC_ID0_LO,
                CTRL_MAC_ID0_LO_MACADDR_7_0);
            macAddr[1U] = HW_RD_FIELD32(SOC_CONTROL_MODULE_REG + CTRL_MAC_ID0_LO,
                CTRL_MAC_ID0_LO_MACADDR_15_8);
            macAddr[2U] = HW_RD_FIELD32(SOC_CONTROL_MODULE_REG + CTRL_MAC_ID0_HI,
                CTRL_MAC_ID0_HI_MACADDR_23_16);
            macAddr[3U] = HW_RD_FIELD32(SOC_CONTROL_MODULE_REG + CTRL_MAC_ID0_HI,
                CTRL_MAC_ID0_HI_MACADDR_23_16);
            macAddr[4U] = HW_RD_FIELD32(SOC_CONTROL_MODULE_REG + CTRL_MAC_ID0_HI,
                CTRL_MAC_ID0_HI_MACADDR_31_24);
            macAddr[5U] = HW_RD_FIELD32(SOC_CONTROL_MODULE_REG + CTRL_MAC_ID0_HI,
                CTRL_MAC_ID0_HI_MACADDR_39_32);
            macAddr[6U] = HW_RD_FIELD32(SOC_CONTROL_MODULE_REG + CTRL_MAC_ID0_HI,
                CTRL_MAC_ID0_HI_MACADDR_47_40);
            break;

        case 2U:
            macAddr[0U] = HW_RD_FIELD32(SOC_CONTROL_MODULE_REG + CTRL_MAC_ID1_LO,
                CTRL_MAC_ID0_LO_MACADDR_7_0);
            macAddr[1U] = HW_RD_FIELD32(SOC_CONTROL_MODULE_REG + CTRL_MAC_ID1_LO,
                CTRL_MAC_ID0_LO_MACADDR_15_8);
            macAddr[2U] = HW_RD_FIELD32(SOC_CONTROL_MODULE_REG + CTRL_MAC_ID1_HI,
                CTRL_MAC_ID1_HI_MACADDR_23_16);
            macAddr[3U] = HW_RD_FIELD32(SOC_CONTROL_MODULE_REG + CTRL_MAC_ID1_HI,
                CTRL_MAC_ID1_HI_MACADDR_31_24);
            macAddr[4U] = HW_RD_FIELD32(SOC_CONTROL_MODULE_REG + CTRL_MAC_ID1_HI,
                CTRL_MAC_ID1_HI_MACADDR_39_32);
            macAddr[5U] = HW_RD_FIELD32(SOC_CONTROL_MODULE_REG + CTRL_MAC_ID1_HI,
                CTRL_MAC_ID1_HI_MACADDR_47_40);
            break;

        default:
            break;
    }
}

#define SYS_MMU_BUFFERABLE      1
#define SYS_MMU_CACHEABLE       2
#define SYS_MMU_SHAREABLE       4
#define SYS_MMU_NO_EXECUTE      8
typedef struct _sys_mmu_entry
{
    void* address;
    /**< Address to be entered in MMU table. */
    unsigned int attributes;
    /**< Attributes of the memory. */
}SYS_MMU_ENTRY;

SYS_MMU_ENTRY applMmuEntries[] = {
    {(void*)0x30000000,SYS_MMU_CACHEABLE},  //QSPI CS0 Maddr1space - Cacheable
    {(void*)0x30100000,SYS_MMU_CACHEABLE},  //QSPI CS0 Maddr1space - Cacheable
    {(void*)0x30200000,SYS_MMU_CACHEABLE},  //QSPI CS0 Maddr1space - Cacheable
    {(void*)0x30300000,SYS_MMU_CACHEABLE},  //QSPI CS0 Maddr1space - Cacheable
    {(void*)0x40300000,0},  //OCMCRAM  - Cacheable
    {(void*)0x44D00000, SYS_MMU_BUFFERABLE},  //PRCM - Non bufferable| Non Cacheable
    {(void*)0x44E00000, SYS_MMU_BUFFERABLE},  //Clock Module, PRM, GPIO0, UART0, I2C0, - Non bufferable| Non Cacheable
    {(void*)0x47900000,SYS_MMU_BUFFERABLE},  //QSPI MMR Maddr0space
    {(void*)0x48000000, SYS_MMU_BUFFERABLE},  //UART1,UART2,I2C1,McSPI0,McASP0 CFG,McASP1 CFG,DMTIMER,GPIO1 -Non bufferable| Non Cacheable
    {(void*)0x48100000,0},  //I2C2,McSPI1,UART3,UART4,UART5, GPIO2,GPIO3,MMC1 - Non bufferable| Non Cacheable
    {(void*)0x48200000, SYS_MMU_BUFFERABLE},  //
    {(void*)0x48300000, SYS_MMU_BUFFERABLE},  //PWM - Non bufferable| Non Cacheable
    {(void*)0x49000000, SYS_MMU_BUFFERABLE},   //EDMA3CC - Non bufferable| Non Cacheable
    {(void*)0x4A000000, SYS_MMU_BUFFERABLE},  //L4 FAST CFG- Non bufferable| Non Cacheable
    {(void*)0x4A100000, SYS_MMU_BUFFERABLE},  //CPSW - Non bufferable| Non Cacheable
    {(void*)0x54400000, SYS_MMU_BUFFERABLE},  //PRU-ICSS0/1 -Bufferable| Non Cacheable | Shareable
    {(void*)0x80000000,SYS_MMU_CACHEABLE},  //QSPI CS0 Maddr1space - Non bufferable| Non Cacheable
    {(void*)0xFFFFFFFF,0xFFFFFFFF}
};

int SDKMMUInit(SYS_MMU_ENTRY mmuEntries[])
{
    unsigned short itr = 0;
    Mmu_FirstLevelDescAttrs attrs;

    if(NULL == mmuEntries)
        return -1;

    Mmu_disable();

    Mmu_initDescAttrs(&attrs);

    attrs.type = Mmu_FirstLevelDesc_SECTION;
    attrs.domain = 0;
    attrs.imp = 1;
    attrs.accPerm = 3;

    for(itr = 0 ; mmuEntries[itr].address != (void*)0xFFFFFFFF ; itr++)
    {
        attrs.bufferable = ((mmuEntries[itr].attributes) & SYS_MMU_BUFFERABLE) && 1 ;
        attrs.cacheable  = ((mmuEntries[itr].attributes) & SYS_MMU_CACHEABLE) && 1;
        if (!attrs.bufferable && !attrs.cacheable)
            attrs.tex = 0; //tex is initialized to 1 and need this to force strongly ordered
        attrs.shareable  = ((mmuEntries[itr].attributes) & SYS_MMU_SHAREABLE) && 1;
        attrs.noexecute  = ((mmuEntries[itr].attributes) & SYS_MMU_NO_EXECUTE) && 1;
        Mmu_setFirstLevelDesc((Ptr)(mmuEntries[itr].address), (Ptr)(mmuEntries[itr].address) , &attrs);  // PWM
    }
    Mmu_enable();
    return 0;
}

void PRUSSDRVPruInit(void)
{
    UInt32 baseaddr;
    PRUICSS_HwAttrs      const    *hwAttrs;

    hwAttrs = pruIcss1Handle->hwAttrs;
    baseaddr = hwAttrs->baseAddr;

	// clear TX RAM located in ICSS PRU0 data RAM
	memset((void *)(baseaddr + PRU_ICSS_DATARAM(0)), 0, (8*1024));
	// clear RX RAM located in ICSS PRU0 data RAM
	memset((void *)(baseaddr + PRU_ICSS_DATARAM(1)), 0, (8*1024));
	// clear ICSS shared RAM
	memset((void *)(baseaddr + PRU_ICSS_SHARED_RAM), 0, (12*1024));
	// clear L3 memory
	memset((void *)0x40300000, 0, (256*1024));

	// Enable OCP master ports for ICSS1
	PRUICSS_enableOCPMasterAccess(pruIcss1Handle);

	HWREG(baseaddr + PRU_ICSS_CFG + HW_ICSS_CFG_GPCFG0) = \
			(3 << HW_ICSS_CFG_GPCFG0_PRU0_GPI_MODE_SHIFT);

	HWREG(baseaddr + PRU_ICSS_CFG + HW_ICSS_CFG_GPCFG1) = \
			(3 << HW_ICSS_CFG_GPCFG0_PRU0_GPI_MODE_SHIFT);

	HWREG(baseaddr + PRU_ICSS_CFG + HW_ICSS_CFG_MII_RT) = 0 | \
			  (1 << HW_ICSS_CFG_MII_RT_EVENT_EN_SHIFT);

	HWREG(baseaddr + PRU_ICSS_CFG + HW_ICSS_CFG_PIN_MX) = 0 | \
				  (0 << HW_ICSS_CFG_PIN_MX_MUX_SEL_SHIFT);

	HWREG(baseaddr + PRU_ICSS_CFG + HW_ICSS_CFG_SPP) = 0 | \
			  (1 << HW_ICSS_CFG_SPP_XFR_SHIFT_EN_SHIFT);

	// Enable PRU cycle counter for TX scheduler
	HWREG(baseaddr + PRU_ICSS_PRU_CTRL(0) + HW_ICSS_PRU_CTRL_CONTROL) |= (1 << 3);
	HWREG(baseaddr + PRU_ICSS_PRU_CTRL(1) + HW_ICSS_PRU_CTRL_CONTROL) |= (1 << 3);

	// set C28 to PRU-ICSS shared memory 0x10000
	HWREG(baseaddr + PRU_ICSS_PRU_CTRL(0) +  HW_ICSS_PRU_CTRL_CTPPR0) = (0x0000<<16)| 0x0100;
	HWREG(baseaddr + PRU_ICSS_PRU_CTRL(0) +  HW_ICSS_PRU_CTRL_CTPPR1) = (0x0000<<16)| 0x3000;
	// set C30 to OCMC memory at 0x40300000
	HWREG(baseaddr + PRU_ICSS_PRU_CTRL(1) +  HW_ICSS_PRU_CTRL_CTPPR0) = (0x0000<<16)| 0x0100;
	HWREG(baseaddr + PRU_ICSS_PRU_CTRL(1) +  HW_ICSS_PRU_CTRL_CTPPR1) = (0x0000<<16)| 0x3000;

	// RXCFG and TXCFG configuration
	HWREG(baseaddr + PRU_ICSS_MIIRT + HW_ICSS_MII_RT_CFG_RXCFG0) =\
			(1<<HW_ICSS_MII_RT_CFG_RXCFG0_RX_ENABLE_SHIFT) |\
			(0<<HW_ICSS_MII_RT_CFG_RXCFG0_RX_CUT_PREAMBLE_SHIFT) |\
			(0<<HW_ICSS_MII_RT_CFG_RXCFG0_RX_MUX_SEL_SHIFT) |\
			(1<<HW_ICSS_MII_RT_CFG_RXCFG0_RX_L2_EN_SHIFT) |\
			(0<<HW_ICSS_MII_RT_CFG_RXCFG0_RX_AUTO_FWD_PRE_SHIFT);
	HWREG(baseaddr + PRU_ICSS_MIIRT + HW_ICSS_MII_RT_CFG_RXCFG1) =\
			(1<<HW_ICSS_MII_RT_CFG_RXCFG1_RX_ENABLE_SHIFT) |\
			(0<<HW_ICSS_MII_RT_CFG_RXCFG1_RX_CUT_PREAMBLE_SHIFT) |\
			(1<<HW_ICSS_MII_RT_CFG_RXCFG1_RX_MUX_SEL_SHIFT) |\
			(1<<HW_ICSS_MII_RT_CFG_RXCFG1_RX_L2_EN_SHIFT) |\
			(0<<HW_ICSS_MII_RT_CFG_RXCFG1_RX_AUTO_FWD_PRE_SHIFT);
	HWREG(baseaddr + PRU_ICSS_MIIRT + HW_ICSS_MII_RT_CFG_TXCFG0) =\
			(1<<HW_ICSS_MII_RT_CFG_TXCFG0_TX_ENABLE_SHIFT) |\
			(0<<HW_ICSS_MII_RT_CFG_TXCFG0_TX_AUTO_PREAMBLE_SHIFT) |\
			(0<<HW_ICSS_MII_RT_CFG_TXCFG0_TX_EN_MODE_SHIFT) |\
			(1<<HW_ICSS_MII_RT_CFG_TXCFG0_TX_MUX_SEL_SHIFT) |\
			(1<<HW_ICSS_MII_RT_CFG_TXCFG0_PRE_TX_AUTO_SEQUENCE_SHIFT) |\
			(0x40<<HW_ICSS_MII_RT_CFG_TXCFG0_TX_START_DELAY_SHIFT);
	HWREG(baseaddr + PRU_ICSS_MIIRT + HW_ICSS_MII_RT_CFG_TXCFG1) =\
			(1<<HW_ICSS_MII_RT_CFG_TXCFG1_TX_ENABLE_SHIFT) |\
			(0<<HW_ICSS_MII_RT_CFG_TXCFG1_TX_AUTO_PREAMBLE_SHIFT) |\
			(0<<HW_ICSS_MII_RT_CFG_TXCFG1_TX_EN_MODE_SHIFT) |\
			(0<<HW_ICSS_MII_RT_CFG_TXCFG1_TX_MUX_SEL_SHIFT) |\
			(1<<HW_ICSS_MII_RT_CFG_TXCFG1_PRE_TX_AUTO_SEQUENCE_SHIFT) |\
			(0x40<<HW_ICSS_MII_RT_CFG_TXCFG1_TX_START_DELAY_SHIFT);
}

int MDIO_Init(t_mdio_params *mdio_conf)
{
	unsigned int temp32, baseaddr;
    PRUICSS_HwAttrs      const    *hwAttrs;

    hwAttrs = pruIcss1Handle->hwAttrs;
    baseaddr = hwAttrs->baseAddr;
	HWREG(baseaddr + PRU_ICSS_MDIO + HW_ICSS_MII_MDIO_CONTROL) = 0x40000000 | mdio_conf->clkdiv; //200/(clkdiv+1)

	Task_sleep(50);

	//check that the pre-configured PHY addresses are available before configuring them.
	//Otherwise, print warning to console and skip the configuration. Return error code
	temp32 = HWREG(baseaddr + PRU_ICSS_MDIO + HW_ICSS_MII_MDIO_ALIVE);
	if((temp32 & (1<<mdio_conf->addr0) && (temp32 & (1<<mdio_conf->addr1))) == 0)
	{
		return(-1);
	}

	// write MDIO configuration to slave configuration registers
	HWREG(baseaddr + PRU_ICSS_SHARED_RAM + MDIO_PHY_CONFIG_OFFSET) = (unsigned int) \
			(mdio_conf->addr0 | \
			(mdio_conf->addr1<<8) | \
			(mdio_conf->link0pol<<16) | \
			(mdio_conf->link1pol<<24));

	if(mdio_conf->enhancedlink_enable)
	{
		// enable fast link PHY0
		HWREG(baseaddr + PRU_ICSS_MDIO + HW_ICSS_MII_MDIO_USERACCESS0) = 0x00 |\
				(1<<31) |\
				(1<<30) |\
				(0<<29) |\
				(0xB<<21) |\
				((mdio_conf->addr0)<<16) |\
				(0x9<<0);

		// enable fast link PHY1
		HWREG(baseaddr + PRU_ICSS_MDIO + HW_ICSS_MII_MDIO_USERACCESS1) = 0x00 |\
				(1<<31) |\
				(1<<30) |\
				(0<<29) |\
				(0xB<<21) |\
				((mdio_conf->addr1)<<16) |\
				(0x9<<0);

		HWREG(baseaddr + PRU_ICSS_MDIO + HW_ICSS_MII_MDIO_USERPHYSEL0) = 0xC0 | mdio_conf->addr0; // Enable LINK_INT and LINKSEL on PHY0
		HWREG(baseaddr + PRU_ICSS_MDIO + HW_ICSS_MII_MDIO_USERPHYSEL1) = 0xC0 | mdio_conf->addr1; // Enable LINK_INT and LINKSEL on PHY1
		HWREG(baseaddr + PRU_ICSS_MDIO + HW_ICSS_MII_MDIO_USERINTMASKED) = (1<<mdio_conf->addr0) | (1<<mdio_conf->addr1); // clear pending interrupts
	}
	else
	{
		HWREG(baseaddr + PRU_ICSS_MDIO + HW_ICSS_MII_MDIO_USERPHYSEL0) = 0x40 | mdio_conf->addr0; // Enable LINK_INT on PHY0
		HWREG(baseaddr + PRU_ICSS_MDIO + HW_ICSS_MII_MDIO_USERPHYSEL1) = 0x40 | mdio_conf->addr1; // Enable LINK_INT on PHY1
		HWREG(baseaddr + PRU_ICSS_MDIO + HW_ICSS_MII_MDIO_USERINTMASKED) = (1<<mdio_conf->addr0) | (1<<mdio_conf->addr1); // clear pending interrupts
	}

	// PHY1 configure FD in parallel detect mode
	while(HWREG(baseaddr + PRU_ICSS_MDIO + HW_ICSS_MII_MDIO_USERACCESS0) & (1<<31));
	HWREG(baseaddr + PRU_ICSS_MDIO + HW_ICSS_MII_MDIO_USERACCESS0) = 0x00 |\
			(1<<31) |\
			(0<<30) |\
			(0<<29) |\
			(0xA<<21) |\
			((mdio_conf->addr0)<<16) |\
			(0x0<<0);
	while((temp32 = HWREG(baseaddr + PRU_ICSS_MDIO + HW_ICSS_MII_MDIO_USERACCESS0)) & (1<<31));
	temp32 |= 1<<5;	// set Force Full-Duplex while working with link partner in forced 100B-TX
	temp32 &= 0xFFFF;
	HWREG(baseaddr + PRU_ICSS_MDIO + HW_ICSS_MII_MDIO_USERACCESS0) = 0x00 |\
			(1<<31) |\
			(1<<30) |\
			(0<<29) |\
			(0xA<<21) |\
			((mdio_conf->addr0)<<16) |\
			(temp32<<0);

	// PHY2 configure FD in parallel detect mode
	while(HWREG(baseaddr + PRU_ICSS_MDIO + HW_ICSS_MII_MDIO_USERACCESS1) & (1<<31));
	HWREG(baseaddr + PRU_ICSS_MDIO + HW_ICSS_MII_MDIO_USERACCESS1) = 0x00 |\
			(1<<31) |\
			(0<<30) |\
			(0<<29) |\
			(0xA<<21) |\
			((mdio_conf->addr1)<<16) |\
			(0x0<<0);
	while((temp32 = HWREG(baseaddr + PRU_ICSS_MDIO + HW_ICSS_MII_MDIO_USERACCESS1)) & (1<<31));
	temp32 |= 1<<5;	// set Force Full-Duplex while working with link partner in forced 100B-TX
	temp32 &= 0xFFFF;
	HWREG(baseaddr + PRU_ICSS_MDIO + HW_ICSS_MII_MDIO_USERACCESS1) = 0x00 |\
			(1<<31) |\
			(1<<30) |\
			(0<<29) |\
			(0xA<<21) |\
			((mdio_conf->addr1)<<16) |\
			(temp32<<0);
	return(0);
}

/**
 * \name	portInit
 * \brief   This function initializes the management data for both ports
 * \param   None
 * \return  0 to tell that all port descriptors are initialized
 */
int portInit(void)
{
	// clear counters
	//clearStatistics();

	// port 0
	switchPort[PORT0].queue.buffer_offset 	   		= PORT2_RXPRU0_BUFFER_ADDR;
	switchPort[PORT0].queue.buffer_desc_offset 		= PORT2_RXPRU0_BUFFER_DESC;
	switchPort[PORT0].queue.queue_desc_offset  		= PORT2_RXPRU0_QUEUE_POINTER;
	switchPort[PORT0].queue.queue_size         		= TOTAL_BUFFER_SIZE;
	switchPort[PORT0].queue.overflow_count_offset	= STAT_RCV_OVERFLOW_P0;

	// port 1
	switchPort[PORT1].queue.buffer_offset 	   		= PORT2_RXPRU1_BUFFER_ADDR;
	switchPort[PORT1].queue.buffer_desc_offset 		= PORT2_RXPRU1_BUFFER_DESC;
	switchPort[PORT1].queue.queue_desc_offset  		= PORT2_RXPRU1_QUEUE_POINTER;
	switchPort[PORT1].queue.queue_size         		= TOTAL_BUFFER_SIZE;
	switchPort[PORT1].queue.overflow_count_offset	= STAT_RCV_OVERFLOW_P1;

	return 0;		// all port descriptors initialized
}

/**
 * \name	RxPktInfo
 * \brief   This function tells which port queue has the packet
 * \param   None
 * \return  0 if port0 queue has the packet, 1 of port1 queue has the packet,
 * 			2 if both ports has packets and -1 if both queues are empty
 */
Int32 RxPktInfo()
{
	Int32 packet_found_flag_port0 = 0;
	Int32 packet_found_flag_port1 = 0;
	Uint16 rd_wr_ptr_store_array[2] = {0};

	// Scan the Port0 to see if it has received any packet
	memcpy( (void*)rd_wr_ptr_store_array, (void*)(PRUSS_SHAREDRAM_BASE + switchPort[0].queue.queue_desc_offset), 4 );
	switchPort[0].queue.rd_ptr = rd_wr_ptr_store_array[0];
	switchPort[0].queue.wr_ptr = rd_wr_ptr_store_array[1];

	if(switchPort[0].queue.wr_ptr != switchPort[0].queue.rd_ptr)
	{
		// Found the packet in the queue
		packet_found_flag_port0 = 1;
	}

	// Scan the Port1 to see if it has received any packet
	memcpy( (void*)rd_wr_ptr_store_array, (void*)(PRUSS_SHAREDRAM_BASE + switchPort[1].queue.queue_desc_offset), 4 );
	switchPort[1].queue.rd_ptr = rd_wr_ptr_store_array[0];
	switchPort[1].queue.wr_ptr = rd_wr_ptr_store_array[1];

	if(switchPort[1].queue.wr_ptr != switchPort[1].queue.rd_ptr)
	{
		// Found the packet in the queue
		packet_found_flag_port1 = 1;
	}

	if (packet_found_flag_port0 == 1 && packet_found_flag_port1 == 1)
	{
		// There is a packet in both queues
		return 2;
	}
	else if (packet_found_flag_port1 == 1)
	{
		// There is a packet in queue for Port1
		return 1;
	}
	else if (packet_found_flag_port0 == 1)
	{
		// There is a packet in queue for Port0
		return 0;
	}
	else
	{
		// There is no packet in either of the queues
		return -1;
	}
}

/**
 * \name	Copy_Packet_PRU2ARM_ESL
 * \brief   This function copies the packet from PRU queue for the respective port to the ARM buffer after
 * 			removing the Preamble and attaching the ESL information (16 bytes) at the end of the packet
 * \param   Port number : 0 for Port0, 1 for Port1
 * \return  Packet length after removing preamble and attaching ESL information
 */
Int32 Copy_Packet_PRU2ARM_ESL (Uint8 port_number)
{
	Uint16	packet_length_with_preamble = 0;
	Uint16	custom_packet_length_with_preamble = 0;
	Uint16	custom_packet_length_first_half = 0;
	Uint16	custom_packet_length_second_half = 0;
	Uint16	updated_pru_rd_ptr = 0;

	// Get length information from the buffer descriptor of respective port
	packet_length_with_preamble = HW_RD_REG16( PRUSS_SHAREDRAM_BASE + switchPort[port_number].queue.buffer_desc_offset + (switchPort[port_number].queue.rd_ptr << 2) + 2 );
	custom_packet_length_with_preamble = BUFFER_SIZE + packet_length_with_preamble;

	// Store the packet without preamble and custom header inside ARM buffer
	//HW_WR_REG32(0x48320194, 1<<10);
	if( (switchPort[port_number].queue.buffer_offset + (switchPort[port_number].queue.rd_ptr << 5) + custom_packet_length_with_preamble) >= (switchPort[port_number].queue.buffer_offset + BUFFER_SIZE*QUEUE_COUNT) )
	{
		custom_packet_length_second_half = (switchPort[port_number].queue.buffer_offset + (switchPort[port_number].queue.rd_ptr << 5) + custom_packet_length_with_preamble) - (switchPort[port_number].queue.buffer_offset + BUFFER_SIZE*QUEUE_COUNT);
		custom_packet_length_first_half = custom_packet_length_with_preamble - custom_packet_length_second_half;

		// To check the case when only the custom header is at the bottom
		if (custom_packet_length_first_half - 32 > 0)
		{
			memcpy( (void*)ARM_buffer, (void*)(L3_OCMC_BASE_ADDRESS + switchPort[port_number].queue.buffer_offset + (switchPort[port_number].queue.rd_ptr << 5) + 32 + 8), custom_packet_length_first_half - 32 - 8 ); // (32) to avoid custom header, (8) to avoid preamble
			memcpy( (void*)(&ARM_buffer[custom_packet_length_first_half - 32 - 8]), (void*)(L3_OCMC_BASE_ADDRESS + switchPort[port_number].queue.buffer_offset), ((custom_packet_length_second_half + 3) & 0xFFFC) ); // (32) to avoid custom header, (8) to avoid preamble, (+3) and then ANDing with 0xFFFC rounds the length to 32bit boundary for memcpy
		}
		// The case when only custom header is at the bottom
		else
		{
			memcpy( (void*)ARM_buffer, (void*)(L3_OCMC_BASE_ADDRESS + switchPort[port_number].queue.buffer_offset + 8), ((custom_packet_length_second_half - 8 + 3) & 0xFFFC) ); // (8) to avoid preamble, (+3) and then ANDing with 0xFFFC rounds the length to 32bit boundary for memcpy
		}
	}
	else
	{
		memcpy( (void*)ARM_buffer, (void*)(L3_OCMC_BASE_ADDRESS + switchPort[port_number].queue.buffer_offset + (switchPort[port_number].queue.rd_ptr << 5) + 32 + 8), ((packet_length_with_preamble - 8 + 3) & 0xFFFC) ); // (+32) to avoid custom header, (+8) to avoid preamble, (-8) to remove preamble length, (+3) and then ANDing with 0xFFFC rounds the length to 32bit boundary for memcpy
	}
	//HW_WR_REG32(0x48320190, 1<<10);

	// Store the upper 8 bytes of beckhoff header inside ARM buffer
	//HW_WR_REG32(0x48320194, 1<<10);
	if (port_number == 0)
	{
		ET2000_Header_Upper_Part_Array[6] = 0x80;	// 0x80 means our Port 0
	}
	else if (port_number == 1)
	{
		ET2000_Header_Upper_Part_Array[6] = 0x40;	// 0x40 means our Port 1
	}
	ET2000_Header_Upper_Part_Array[7] = HW_RD_REG8( PRUSS_SHAREDRAM_BASE + switchPort[port_number].queue.buffer_desc_offset + (switchPort[port_number].queue.rd_ptr << 2) );
	memcpy( (void*)(&ARM_buffer[packet_length_with_preamble - 8]), (void*)ET2000_Header_Upper_Part_Array, 8 );

	// Store the lower 8 bytes (Timestamp information) of beckhoff header inside ARM buffer
	if (port_number == 0)
	{
		memcpy( (void*)(&ARM_buffer[packet_length_with_preamble - 8 + 8]), (void*)timestamp_store_array_port0, 8 );
	}
	else if (port_number == 1)
	{
		memcpy( (void*)(&ARM_buffer[packet_length_with_preamble - 8 + 8]), (void*)timestamp_store_array_port1, 8 );
	}
	//HW_WR_REG32(0x48320190, 1<<10);

	// Update PRU queue Rd Pointer; // Doing it with same logic as done in the PRU firmware for PRU queue Wr Pointer
	updated_pru_rd_ptr = switchPort[port_number].queue.rd_ptr + ((packet_length_with_preamble + BUFFER_SIZE - 1) / BUFFER_SIZE) + 1;
	if ( BUFFER_DESC_COUNT > updated_pru_rd_ptr)
	{
		HW_WR_REG16( (PRUSS_SHAREDRAM_BASE + switchPort[port_number].queue.queue_desc_offset), updated_pru_rd_ptr );
	}
	else
	{
		updated_pru_rd_ptr = updated_pru_rd_ptr - BUFFER_DESC_COUNT;
		HW_WR_REG16( (PRUSS_SHAREDRAM_BASE + switchPort[port_number].queue.queue_desc_offset), updated_pru_rd_ptr );
	}

	return (packet_length_with_preamble + 16 - 8); //Beckhoff info (16), SFD/Preamble (8)
}

/**
 * \name	RxPktTransfer_ESL
 * \brief   This function calls the function RxPktInfo() to find out which port queue has the packet and then based on the
 * 			return value calles the Copy_Packet_PRU2ARM_Beckhoff() to transfer the packet from PRU queue to ARM buffer
 * \param   None
 * \return  Packet length after removing preamble and attaching ESL information or -1 if their is no packet in either queue
 */
Int32 RxPktTransfer_ESL()
{
	Int32 	packet_availablility_code = -1;
	Uint32	timestamp_low_port0 = 0;
	Uint32	timestamp_low_port1 = 0;
	Uint32	timestamp_high_port0 = 0;
	Uint32	timestamp_high_port1 = 0;
	Uint8	port_number;
	Int32 	return_val;

	//HW_WR_REG32(0x48320194, 1<<10);
	packet_availablility_code = RxPktInfo();
	//HW_WR_REG32(0x48320190, 1<<10);

	// Get timestamp information for the packet from queue for port0
	memcpy( (void*)timestamp_store_array_port0, (void*)(L3_OCMC_BASE_ADDRESS + switchPort[0].queue.buffer_offset + (switchPort[0].queue.rd_ptr << 5) + IEP_LOW_COUNTER_OFFSET), 8 );
	timestamp_low_port0 = timestamp_store_array_port0[0];
	timestamp_high_port0 = timestamp_store_array_port0[1];
	// Get timestamp information for the packet from queue for port1
	memcpy( (void*)timestamp_store_array_port1, (void*)(L3_OCMC_BASE_ADDRESS + switchPort[1].queue.buffer_offset + (switchPort[1].queue.rd_ptr << 5) + IEP_LOW_COUNTER_OFFSET), 8 );
	timestamp_low_port1 = timestamp_store_array_port1[0];
	timestamp_high_port1 = timestamp_store_array_port1[1];

	//HW_WR_REG32(0x48320194, 1<<10);
	switch(packet_availablility_code)
	{
		case 0 :
			port_number = 0;
			//HW_WR_REG32(0x48320194, 1<<10);
			return_val = Copy_Packet_PRU2ARM_ESL(port_number);
			//HW_WR_REG32(0x48320190, 1<<10);
			break;
		case 1 :
			port_number = 1;
			//HW_WR_REG32(0x48320194, 1<<10);
			return_val = Copy_Packet_PRU2ARM_ESL(port_number);
			//HW_WR_REG32(0x48320190, 1<<10);
			break;
		case 2 :
			if (timestamp_high_port0 < timestamp_high_port1)
			{
				port_number = 0;
				//HW_WR_REG32(0x48320194, 1<<10);
				return_val = Copy_Packet_PRU2ARM_ESL(port_number);
				//HW_WR_REG32(0x48320190, 1<<10);
			}
			else if (timestamp_high_port0 == timestamp_high_port1)
			{
				if (timestamp_low_port0 <= timestamp_low_port1)
				{
					port_number = 0;
					//HW_WR_REG32(0x48320194, 1<<10);
					return_val = Copy_Packet_PRU2ARM_ESL(port_number);
					//HW_WR_REG32(0x48320190, 1<<10);
				}
				else
				{
					port_number = 1;
					//HW_WR_REG32(0x48320194, 1<<10);
					return_val = Copy_Packet_PRU2ARM_ESL(port_number);
					//HW_WR_REG32(0x48320190, 1<<10);
				}
			}
			else
			{
				port_number = 1;
				//HW_WR_REG32(0x48320194, 1<<10);
				return_val = Copy_Packet_PRU2ARM_ESL(port_number);
				//HW_WR_REG32(0x48320190, 1<<10);
			}
			break;
		default :
			return_val = -1;
			break;
	}
	//HW_WR_REG32(0x48320190, 1<<10);
	return return_val;
}

/**
 * \name	Diagnostics_FreeSpace
 * \brief   This function calculates the free space available inside the port queue based on the read and write pointer values for that queue
 * \param   Read and write pointers for the queue whom free space is to be calculated
 * \return  Available space in terms of 32 byte blocks, (1 means 32 bytes are available)
 */
int Diagnostics_FreeSpace(Uint16 rd_pos, Uint16 wr_pos)
{
	int available_space = 0;

	if (rd_pos > wr_pos)
	{
		available_space = rd_pos - wr_pos;
	}
	else
	{
		available_space = rd_pos + (QUEUE_COUNT - wr_pos);
	}

	return available_space;
}

/**
 *  \name main
 *  \brief Main Function
 *  \param none
 *  \return none
 */
int main()
{
    /* Call board init functions */
    Board_initCfg boardCfg;
    Task_Params taskParams;
    unsigned int temp32;

    SDKMMUInit(applMmuEntries);

    pruIcss1Handle = PRUICSS_create(pruss_config, PRUICSS_INSTANCE);

    boardCfg = BOARD_INIT_PINMUX_CONFIG |  BOARD_INIT_ICSS_PINMUX |
    		BOARD_INIT_MODULE_CLOCK | BOARD_INIT_UART_STDIO;
    Board_init(boardCfg);

    PINMUXModuleConfig(CHIPDB_MOD_ID_CPSW, 0U, NULL);

	/* Chip configuration MII/RMII selection */
    CpswPortMacModeSelect(1, ETHERNET_MAC_TYPE_RGMII);
    CpswPortMacModeSelect(2, ETHERNET_MAC_TYPE_RGMII);

    Task_Params_init(&taskParams);
    taskParams.priority = 2;
    taskParams.stackSize = 0x1400;
    main_task = Task_create (TaskFxn, &taskParams, NULL);

    Task_Params_init(&taskParams);
    taskParams.priority = 1;
    taskParams.stackSize = 0x1400;
    main_task = Task_create (TaskDiagnostics, &taskParams, NULL);

    NIMUDeviceTable[nimu_device_index++].init =  &CpswEmacInit ;
    NIMUDeviceTable[nimu_device_index].init =  NULL ;

    PRUICSS_pinMuxConfig(pruIcss1Handle, 0x0);   // PRUSS pinmuxing
    //Disable PRUs - This is to ensure PRUs are not running when application is not initialized
    PRUICSS_pruDisable(pruIcss1Handle, 0);
    PRUICSS_pruDisable(pruIcss1Handle, 1);

    //gpio4_10
    // 9D8h CTRL_CONF_CAM1_VD
    // control module 0x44E10000
    temp32 = HW_RD_REG32(0x48320134);
    temp32 &=~(1<<10);
    HW_WR_REG32(0x48320134, temp32);

    BIOS_start();

    return -1;
}

// Dummy packet for debugging purpose
unsigned int udp19[] = { 0x7A020100, 0x94D49EA3,  0xABAE3BA1, 0x00450008,
            0x04002F00, 0x11400000,  0xA8C05AF7, 0xA8C00401,
            0x07000B01, 0x1B00B512,  0x14139F50, 0x18171615,
            0x1C1B1A19, 0x201F1E1D,  0x24232221, 0x00000025,
            0xeeeeeeee};

/**
 *  \name TaskFxn
 *  \brief Task which do EIP initialization
 *  \param a0
 *  \param a1
 *  \return none
 *
 */
void TaskFxn(UArg a0, UArg a1)
{
	int i = 0;
	unsigned int counter = 150;
	Int32 packet_length_in_ARM_buffer = 0;
	unsigned int ndk_sent_packet_length = 0;
	unsigned int ndk_buffer_error = 0;
	unsigned int sock_buf_error = 0;
	Uint8 firmwareLoad_done = FALSE;
	SOCKET Sock;
	PBM_Handle  hPkt = NULL;
	int ret_val;
	//int error_code;
	Uint8* pBuffer = NULL;


	PRUICSS_IntcInitData pruss_intc_initdata = PRUSS_INTC_INITDATA;
	PRUICSS_pruIntcInit(pruIcss1Handle, &pruss_intc_initdata);

	NIMU_log("\n\rSYS/BIOS Real-Time Ethernet Tracer\n\r");

    UART_printf("\nVersion - ");
    UART_printf(IND_PKG_VERSION);

    Board_IDInfo boardInfo;
    Board_getIDInfo(&boardInfo);

    UART_printf("\nBoard name \t: ");
    UART_printf(boardInfo.boardName);

    UART_printf("\n\rChip Revision \t: ");
    UART_printf(boardInfo.version);

    PRUICSS_pruDisable(pruIcss1Handle, 0);
    PRUICSS_pruDisable(pruIcss1Handle, 1);

    PRUICSS_pruReset(pruIcss1Handle, 0);
    PRUICSS_pruReset(pruIcss1Handle, 1);

    if(PRUICSS_pruWriteMemory(pruIcss1Handle,PRU_ICSS_IRAM(0) ,0,
                             (uint32_t *) RTE_TRACER_PRU_ICSS_PRU0,
                             sizeof(RTE_TRACER_PRU_ICSS_PRU0)))
    {
    	if(PRUICSS_pruWriteMemory(pruIcss1Handle,PRU_ICSS_IRAM(1) ,0,
                                 (uint32_t *) RTE_TRACER_PRU_ICSS_PRU1,
                                 sizeof(RTE_TRACER_PRU_ICSS_PRU1)))
        {
    		firmwareLoad_done = TRUE;
        }
    }

    PRUSSDRVPruInit();
	portInit();

	if(MDIO_Init(&mdio_configuration) != 0)
	{
		UART_printf("\n\rPHY address %d or %d not detected", mdio_configuration.addr0, mdio_configuration.addr1);
	   // do{;}while(1);
	}

    if( firmwareLoad_done)
    {
        PRUICSS_pruEnable(pruIcss1Handle, 0);
        PRUICSS_pruEnable(pruIcss1Handle, 1);
    }

	// Open the file session
	fdOpenSession(TaskSelf());

	// Sending the dummy packet, just for debugging
	Sock = socket(AF_RAWETH, SOCK_RAWETH, 0x4A58);
	i = 1;
	setsockopt(Sock, SOL_SOCKET, SO_IFDEVICE, &i, sizeof(i));
	send(Sock, &udp19[0], sizeof(udp19), 0);
	Task_sleep(1);

	do{
	    //HW_WR_REG32(0x48320194, 1<<10);
	    packet_length_in_ARM_buffer = RxPktTransfer_ESL();
	    //HW_WR_REG32(0x48320190, 1<<10);

	    pBuffer = ARM_buffer;					//added 21-9-2016

	    if (packet_length_in_ARM_buffer != -1)
	    {
	    	//HW_WR_REG32(0x48320194, 1<<10);
	    	ndk_sent_packet_length = send(Sock, (Uint8 *)pBuffer, packet_length_in_ARM_buffer, 0);
	    	//HW_WR_REG32(0x48320190, 1<<10);
	    	counter--;
	    	if (ndk_sent_packet_length == -1)
	    	{
	    		// Incrementing counter for how many times the ndk buffer failed to store the sending packet
	    		//error_code = fdError();
	    		ndk_buffer_error++;
	    	}

	    	//sendncfree(hPkt);					//added 21-9-2016
	    }

	    if (counter == 0)
	    {
	    	Task_yield();
	    	//Task_sleep(1);
	    }
	    if (packet_length_in_ARM_buffer == -1)
	    {
	    	Task_yield();
	    	//Task_sleep(1);
	    }
	  }while(1);
}

/**
 *  \name 	TaskDiagnostics
 *  \brief 	This task function is to get the diagnostics data, this stores the diagnostic data like
 *  		free space and overflow count in a point in time for each port in separate arrays
 *  \param a0
 *  \param a1
 *  \return none
 *
 */
void TaskDiagnostics(UArg a0, UArg a1)
{
	int i = 0;
	Uint16	rd_ptr_pru0_queue = 0;
	Uint16	wr_ptr_pru0_queue = 0;
	Uint16 	rd_ptr_pru1_queue = 0;
	Uint16 	wr_ptr_pru1_queue = 0;

	do{
		rd_ptr_pru0_queue = HW_RD_REG16(PRUSS_SHAREDRAM_BASE + switchPort[0].queue.queue_desc_offset);
		wr_ptr_pru0_queue = HW_RD_REG16(PRUSS_SHAREDRAM_BASE + switchPort[0].queue.queue_desc_offset + 2);

		empty_space_diagnostic_array_pru0[i] = Diagnostics_FreeSpace(rd_ptr_pru0_queue, wr_ptr_pru0_queue);
		overflow_diagnostic_array_pru0[i] = HW_RD_REG16(PRUSS_SHAREDRAM_BASE + switchPort[0].queue.overflow_count_offset);

		rd_ptr_pru1_queue = HW_RD_REG16(PRUSS_SHAREDRAM_BASE + switchPort[1].queue.queue_desc_offset);
		wr_ptr_pru1_queue = HW_RD_REG16(PRUSS_SHAREDRAM_BASE + switchPort[1].queue.queue_desc_offset + 2);

		empty_space_diagnostic_array_pru1[i] = Diagnostics_FreeSpace(rd_ptr_pru1_queue, wr_ptr_pru1_queue);
		overflow_diagnostic_array_pru1[i] = HW_RD_REG16(PRUSS_SHAREDRAM_BASE + switchPort[1].queue.overflow_count_offset);
		i++;
		if (i == DIAGNOSTIC_ARRAYS_SIZE)
		{
			i = 0;
		}
		Task_sleep(20);
	}while(1);
}
