/*
* Real-time Ethernet on PRU-ICSS
*
* Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
*
*    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

#ifndef __RTE_DRV_H
#define __RTE_DRV_H

typedef struct {
 Uint32 bus_cycle_time;
 Uint32 rt_window_open;
 Uint32 nrt_window_open;
 Uint32 nrt_last_tx;
 Uint32 host_event;
 } t_bus_config;

typedef struct {
 Uint16 clkdiv;	// MDIO clock = 200/(clkdiv+1)
 Uint8 addr0;
 Uint8 addr1;
 Uint8 link0pol; //1: Active Low 0: Active High
 Uint8 link1pol; //1: Active Low 0: Active High
 Uint8 enhancedlink_enable;
} t_mdio_params;

typedef struct {
 unsigned int tts_starttime;
 unsigned short tts_frame_offset;
 unsigned short tts_frame_length;
} t_tts_descriptor_params;

#define MAXMTU	1518		// this is IP max (incl header + VLAN TAG..., no CRC)
//#define L3_OCMC_BASE_ADDRESS			0x40300000

// packet IF structures - here for PRU-ICSS switch
// statistics per queue
typedef struct queStats
{
	Uint32 rawCount;
	Uint32 errCount;
} t_queStats;

// queues
typedef struct queParams
{
	Uint16 		buffer_offset;
	Uint16 		buffer_desc_offset;
	Uint16 		queue_desc_offset;
	Uint16 		queue_size;
	Uint16		rd_ptr;
	Uint16		wr_ptr;
	Uint16		overflow_count_offset;
	t_queStats 	qStat;
} t_queParams;

//#define NUMQUEUES 1					// changed to 1 from 2 on 26-7-2016
//#define QUEUE0 0
//#define QUEUE1 1

// ports
typedef struct portParams
{
	t_queParams		queue;
	Uint32			rawCount;				// statistics   - raw
	Uint32			errCount;				//				- errored
	Uint32			port1Count;
	Uint32			port2Count;
} t_portParams;


//read/write pointer of port queues
#define PORT0_FWD_QUEUE_POINTER				0x0100	// handled by PRU1
#define PORT0_HOST_QUEUE_POINTER			0x0104	// handled by PRU1
#define PORT1_FWD_QUEUE_POINTER				0x0108	// handled by PRU0
#define PORT1_HOST_QUEUE_POINTER			0x010C	// handled by PRU0
#define PORT2_PORT0_QUEUE_POINTER			0x0110	// handled by PRU0
#define PORT2_PORT1_QUEUE_POINTER			0x0114	// handled by PRU1

//frame descriptor base address: 32 frame descriptors per queue
#define BUFFER_DESC_COUNT					500
#define PORT0_FWD_BUFFER_DESCR				0x0120
#define PORT0_HOST_BUFFER_DESCR				(PORT0_FWD_BUFFER_DESCR + BUFFER_DESC_COUNT*4)
#define PORT1_FWD_BUFFER_DESCR				(PORT0_HOST_BUFFER_DESCR + BUFFER_DESC_COUNT*4)
#define PORT1_HOST_BUFFER_DESCR				(PORT1_FWD_BUFFER_DESCR + BUFFER_DESC_COUNT*4)
#define PORT2_PRU0_BUFFER_DESCR				(PORT1_HOST_BUFFER_DESCR + BUFFER_DESC_COUNT*4)
#define PORT2_PRU1_BUFFER_DESCR				(PORT2_PRU0_BUFFER_DESCR + BUFFER_DESC_COUNT*4)

//frame queue base address in OCMC memory
#define BUFFER_SIZE							32		//size of each queue entry in bytes
#define QUEUE_COUNT							BUFFER_DESC_COUNT	//number of queue entries
#define PORT0_FWD_BUFFER_ADDR				0x0000
#define PORT0_HOST_BUFFER_ADDR				(PORT0_FWD_BUFFER_ADDR +  BUFFER_SIZE*QUEUE_COUNT)
#define PORT1_FWD_BUFFER_ADDR				(PORT0_HOST_BUFFER_ADDR +  BUFFER_SIZE*QUEUE_COUNT)
#define PORT1_HOST_BUFFER_ADDR				(PORT1_FWD_BUFFER_ADDR +  BUFFER_SIZE*QUEUE_COUNT)
#define PORT2_PRU0_BUFFER_ADDR				(PORT1_HOST_BUFFER_ADDR +  BUFFER_SIZE*QUEUE_COUNT)
#define PORT2_PRU1_BUFFER_ADDR				(PORT2_PRU0_BUFFER_ADDR +  BUFFER_SIZE*QUEUE_COUNT)

#define PORT0 0									// Port 0: MII 0     (rx)
#define PORT1 1									// Port 1: MII 1	 (rx)
#define PORT2 2									// Port 2: Host side (tx)

void rte_SetDeviceAddress(unsigned char device_addr);
void rte_SetBusConfiguration(t_bus_config *BusConf);
void rte_StartStopIepTimer(unsigned int startstop);

#endif

